% QuamboInput.m %
% Xiaofeng Qian, MIT (Jan. 4, 2006) %


%%%%%%%%%%%%%%%%%%  Input data  %%%%%%%%%%%%%%%%%%%%%%%%

% last slash symbol "/" is necessary for  pointing directory correctly %
Input.DFTDataDir = ['./'];
Input.saveDir    = ['./QUAMBO/'];


saveDir = Input.saveDir;

%%%%%%%%%%%%%%%%%%   If one needs to Load old QUAMBO Data  %%%%%%%%%%%%%%%%%%%%%%%
Input.LoadOldQuamboDataOrNOT = 0;      % 0-No, 1-Yes 
Input.QUAMBOData.Filename    = [saveDir '/Quambo.TBdata.mat'];


%%%%%%%%%%%%%%%%%%   If one needs to start a new QUAMBO calculation  %%%%%%%%%%%%%
% ============== Input: DFT data and pseudopotential data ================ %
Input.DFTType            = 'VASP';
Input.DFTNetCDF          = [Input.DFTDataDir '/vaspdata.nc'];
Input.PseudopotList      = ...
    {'Fe'; '/usr/local/dacapo/psp/Fe/PW91/Fe_us_gga_soft.pseudo'};

% ============== Input: Parameters for scaling orbitals ================ %
% scaling pseudo-atomic orbitals in the order of [ s p d f g h] %
Input.ConvertPsp.OrbitalScaling           = [ 0  0.5  0.5  0.];
Input.ConvertPsp.AddOrDeleteOrbital       = [0];       % 0: No, 1: add , -1: delete %
Input.ConvertPsp.AddOrbital_L             = [1];
Input.ConvertPsp.DeleteOrbitalIndex       = [3];
Input.ConvertPsp.ShowScaledOrbital        = 1; % 0-No, 1-Yes 

% ==== Input: define Fermi level shift to grab more exact eigen states ==== %
Input.CombineUnoccupiedBV.EFermiShift         = 2.0; % eV % 
Input.CombineUnoccupiedBV.MaxConditionNumber  = 50;
Input.CombineUnoccupiedBV.TotalPlaneWaveBands = 25; % 0: include all DFT PW bands%

%%%%%%%%%%%%%%%%%%   Analysis  %%%%%%%%%%%%%%%%%%%%%%%
% ============== Tight Binding ============== %
Input.Tightbinding.CalculateOrNot = 1;
Input.Tightbinding.DataFilename = ...
    [saveDir '/Quambo.TBdata.mat'];

% ============== Threshold for Generalized eigenvalue problem ============== %
Input.GenEig.Threshold = 1e-2;

% ============== Analysis: Band Structure ============ %
Input.BandStructure.CalculateOrNot   = 1;
Input.BandStructure.Grid             = 50;
% Input.BandStructure.CrystalStructure = ['BCC']; %
% One can define own band structure K-path %
Input.BandStructure.CrystalStructure = ['DEF'];
Input.BandStructure.DefinedKPathName = ['GHNPGN'];
Input.BandStructure.DefinedKPath     = ...
    [    0     0     0; ...
        -1/2   1/2   1/2; ...
         0     0     1/2; ...
         1/4   1/4   1/4; ...
         0     0     0 ; ...
         0     0     1/2;
    ];
Input.BandStructure.Filename         = [saveDir '/Quambo.BandStructure.mat'];
Input.BandStructure.RCutoffYesOrNo   = 1; % 0-No, 1-Yes 
Input.BandStructure.RCutoff          = 10; % A 

Input.PlotBandStructure.EnergyRange  = [-15 20];  % eV
Input.PlotBandStructure.Filename     = [saveDir '/Quambo.BandStructure.eps'];

% ============== Analysis: Density of States ============ %
Input.DOS.CalculateOrNot    = 1;
Input.DOS.BroadeningWidth   = 0.4;
Input.DOS.EnergyRange       = [-20 40];
Input.DOS.EnergyGridPoints  = 400;
Input.DOS.Filename          = [saveDir '/Quambo.DOS.mat'];
Input.PDOS.Filename          = [saveDir '/Quambo.PDOS.mat'];

% ============== Analysis: Fermi Surface ============ %
Input.Fermisurface.CalculateOrNot = 1;
Input.Fermisurface.Grid     =  [16 16 16];
Input.Fermisurface.Filename = [saveDir '/Quambo.FermiSurface.bxsf'];

% ========= Analysis: Mulliken Charge ======== %
Input.Mulliken.CalculateOrNot     = 0;
Input.Mulliken.MatlabDataFilename = [saveDir '/Quambo.Mulliken.mat'];
Input.Mulliken.EpsFilename        = [saveDir '/Quambo.Mulliken.eps'];

% ============== Analysis: Quambo Orbital and related Data output ============ %
Input.OutputQuambo.CalculateOrNot  = 1;
Input.OutputQuambo.SizeOfBV        = [ 3 3 3 ];
Input.OutputQuambo.OrbitalFilename = [saveDir '/Quambo.Orbital.xsf'];
Input.OutputQuambo.DataFilename    = [saveDir '/Quambo.DataforBond.mat'];
Input.OutputAtomicOrbital.YesorNo  = [ 0 ];  % 0 -No, 1 -Yes % 
Input.OutputQuambo.AllAtom  = 1;  % 0 -No, 1 -Yes % 
Input.OutputQuambo.AtomList = [1];

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

Input.SaveFinalQUAMBODataOrNot = 1;
Input.SaveFinalQUAMBO.Filename = [saveDir '/Quambo.QuamboFinalData.mat'];
