% CheckTBOverlap.m %
% Check if QuickTBOverlap has generated correct overlap matrix: %
% <quambo_0i| H |quambo_nj> and  <quambo_0i| S |quambo_nj> %
% Compare with EnkTB from CheckTightBinding.m %

function [EnkQuambo, EnormdiffTB] = CheckTBOverlap(QMB, Input);
  fprintf (1, '[CheckTBOverlap] Check Tightbinding Hamiltonian from TB H and S matrix\n');

format long;


GenEigThreshold = Input.GenEig.Threshold;
Eshift          = Input.CombineUnoccupiedBV.EFermiShift;

TBHamOverlap   = QMB.TBHamOverlap;
TBSOverlap     = QMB.TBSOverlap;
number_of_spin = QMB.nSpins;

EnkExact = QMB.Enk_PW;
IBZKMax  = QMB.IBZKMax;
atomtype = QMB.atomtype;
atomMax  = QMB.atomMax;
NHomo    = zeros(IBZKMax, number_of_spin);
Rdim     = QMB.Rdim;
IBZK     = QMB.IBZK;
Efermi   = QMB.Efermi;
orbMax   = QMB.orbMax;
H        = QMB.H;


EnormdiffTB = 0;
naddband = 0;
% Hdim = totalorbital; %

id = 0;
for ixx = -Rdim(1) : Rdim(1),
  ix = ixx + Rdim(1) + 1;
  for iyy = -Rdim(2) : Rdim(2),
    iy = iyy + Rdim(2) + 1;
    for izz = -Rdim(3) : Rdim(3),
      iz = izz + Rdim(3) + 1;
      
      id = id + 1;
      RCellID(id, :)  = [ix iy iz];
      RCellPos(id, :) = [ixx iyy izz];
      
    end;
  end;
end;
TotalCell = size(RCellID,1);



Hdim = size(TBHamOverlap,2);
for ispin = 1 : number_of_spin,
  for ibz = IBZKMax : -1 : 1,
    kvector = IBZK(ibz,:)';
    
    lumo = min( find( EnkExact(ibz,ispin, :) > ...
                      Efermi+Eshift) );
    
    homo = lumo - 1;
    homo = homo + naddband;
    NHomo(ibz, ispin) = homo;
    
    HH = zeros(Hdim, Hdim);
    SS = zeros(Hdim, Hdim);
    
    OrbX  = 0;
    OrbXX = 0;
    for iatom = 1 : atomMax,
      itype = atomtype(iatom);
      for iorb = 1 : orbMax(itype);
        OrbXX = OrbX + iorb;
        
        OrbY = 0;
        OrbYY = 0;
        for jatom = 1 : atomMax,
          jtype = atomtype(jatom);
          for jorb = 1 : orbMax(jtype);
            OrbYY = OrbY + jorb;
            
            HH(OrbXX, OrbYY) = ...
                HH(OrbXX, OrbYY) + ...
                sum( squeeze(TBHamOverlap(ispin, OrbXX, OrbYY, :)) .* ...
                     exp( i *( RCellPos(:,:) * H ) * kvector) );
            
            SS(OrbXX, OrbYY) = ...
                SS(OrbXX, OrbYY) + ...
                sum( squeeze(TBSOverlap(ispin, OrbXX, OrbYY, :)) .*  ...
                     exp( i *( RCellPos(:,:) * H ) * kvector) );
            
          end;
          OrbY = OrbY + orbMax(jtype);
        end;
        
      end;
      OrbX = OrbX + orbMax(itype);
    end;
    
%    [vv, dd] = eig(HH, SS); %
    dd = diag(GenEig(HH, SS, GenEigThreshold));
    %   [ibz  NHomo(ibz,ispin) ] %
    EnkQ = sort(real(diag(dd)));
    
    EnkQuambo(ibz,ispin,1:length(dd)) = EnkQ;
    
    ta = squeeze( EnkQ   ); % TB
    tc = squeeze( EnkExact(ibz, ispin, 1:Hdim) ); % dacapo
    
    if ((length(ta) > 0) && (~isempty(ta) ) && (NHomo(ibz,ispin)>0) ),
      normdiff = norm(ta(1:NHomo(ibz,ispin))-tc(1:NHomo(ibz,ispin))) / ...
          norm( tc(1:NHomo(ibz,ispin)) ) ;
      
      if (normdiff > 1e-4),
        [ibz homo]
%         ta-Efermi %
%         tc-Efermi %
        [ta-Efermi tc(1:length(ta))-Efermi ta-tc(1:length(ta))]
        fprintf(1,'[CheckTBOverlap]  ibz=%d,  normdiff_ratio=%f\n', ibz, normdiff);
        pause;
      end;
      
      EnormdiffTB = EnormdiffTB + normdiff;
    end;
    
  end;
end;

fprintf (1, ['\nEnergy norm difference ratio =  ' num2str(EnormdiffTB) '\n']);
fprintf (1, '\n');

format short;
