% vasp2net.m %
loadvaspdata;



% open netcdf file %
nc = netcdf('vaspdata.nc','clobber');
nc.description = 'VASP data for Quambo';                   % Global attributes.
nc.author = 'Liang Qi';
nc.date = date;

% define dimensions %
nc('dim1')=1;
nc('dim2')=2;
nc('dim3')=3;
nc('dim6')=6;
nc('dim7')=7;
nc('real_complex')=2;
nc('number_ionic_steps')=1;
nc('number_of_dynamic_atoms')=NIONS;
nc('number_of_spin')=ISPIN;
nc('number_of_bands')=NB;
nc('softgrid_dim1')=NGX;
nc('softgrid_dim2')=NGY;
nc('softgrid_dim3')=NGZ;
nc('hardgrid_dim1')=NGX;
nc('hardgrid_dim2')=NGY;
nc('hardgrid_dim3')=NGZ;
nc('number_of_rotmatrix')=NROTK;
nc('number_BZ_kpoints')=FULLKPT;
nc('number_IBZ_kpoints')=NKPT;
nc('number_plane_waves')=MAXNPL;
nc('max_projectors_per_atom')=MAXLM;
nc('number_of_species')=NTYP;

% 05/04/2006 %
nc('max_L_of_all_atom') = MAXL;
nc('length_of_betak') = NPSPNL;


% define variables %
% added %
nc{'AtomSymbol'} = ncchar('number_of_species','dim2');
nc{'DynamicAtomSpecies'}  = {'number_of_dynamic_atoms', 'dim2'};     

% 05/04/2006 %
nc{'ReciprocalProjectorK'}   = {'number_of_species','max_L_of_all_atom','length_of_betak'};
nc{'ReciprocalProjectorK_L'} = {'number_of_species','max_L_of_all_atom'};
nc{'ReciprocalProjectorKGMax'} = {'number_of_species'};
nc{'AugmentationQij'}={'number_of_species','max_projectors_per_atom','max_projectors_per_atom'};


nc{'PlaneWaveCutoff'} = 'dim1';
nc{'UnitCell'} = {'number_ionic_steps', 'dim3', 'dim3'};
nc{'KpointSetup'} = 'dim3';
nc{'UseSymmetry'} = 'dim7';
nc{'DynamicAtomPositions'} = {'number_ionic_steps', 'number_of_dynamic_atoms', 'dim3'}; 
nc{'DynamicAtomPositions'}.units = 'Angstrom';                          
nc{'LatticeRotationMatrix'} = {'number_of_rotmatrix', 'dim3', 'dim3'};
nc{'BZKpoints'} = {'number_BZ_kpoints', 'dim3'};      %
nc{'IBZKpoints'} = {'number_IBZ_kpoints', 'dim3'};  
nc{'KpointWeight'}  = {'number_IBZ_kpoints'};
nc{'KpointBZtoIBZ'} = {'number_BZ_kpoints'};          % 
nc{'NumberPlaneWavesKpoint'} = {'number_IBZ_kpoints'};
nc{'EigenValues'} = {'number_ionic_steps', 'number_IBZ_kpoints', 'number_of_spin', 'number_of_bands'};
nc{'OccupationNumbers'} = {'number_ionic_steps', 'number_IBZ_kpoints', 'number_of_spin', 'number_of_bands'};
nc{'FermiLevel'}  = {'dim1'}; 
nc{'WaveFunction'} = {'number_IBZ_kpoints', 'number_of_spin', 'number_of_bands', 'number_plane_waves', 'real_complex'};
nc{'WaveFunctionFFTindex'} = {'number_IBZ_kpoints','number_plane_waves'};
nc{'TypeNLProjectorl'} = {'max_projectors_per_atom', 'number_of_dynamic_atoms'};%
nc{'TypeNLProjectorm'} = {'max_projectors_per_atom', 'number_of_dynamic_atoms'};%
nc{'NumberOfNLProjectors'} = {'number_of_dynamic_atoms'};%

nc{'AtomSymbol'}(:)=spec;
nc{'UnitCell'}(:) = reshape(H, 1,3,3);
nc{'KpointSetup'}(:) = floor(IKPT(1,2:4)+1e-4);
if ( FULLKPT~=size(VKPT,1) ),
  nc{'UseSymmetry'}(:) = 'Maximum';
else
  nc{'UseSymmetry'}(1:3) = 'Off';
end;
nc{'DynamicAtomPositions'}(:)=reshape(POS,1,NIONS,3);
nc{'DynamicAtomSpecies'}(:)=SPEC;
nc{'LatticeRotationMatrix'}(:) = IGROP;
nc{'IBZKpoints'}(:)=VKPT;
nc{'KpointWeight'}(:)=WTKPT;

nc{'NumberPlaneWavesKpoint'}(:)=NPL;
nc{'EigenValues'}(:) = reshape(EIG,1,NKPT,ISPIN,NB);
nc{'OccupationNumbers'}(:) = reshape(FERWEI,1,NKPT,ISPIN,NB);

% 05/04/2006 %
nc{'ReciprocalProjectorK'}(:) = QPROJK;
nc{'ReciprocalProjectorK_L'}(:) = PROJ_L;
nc{'ReciprocalProjectorKGMax'}(:) = BETAKGMAX;

nc{'WaveFunctionFFTindex'}(:) =FFTINDEX;
nc{'WaveFunction'}(:) =WAV;

nc{'AugmentationQij'}(:) =QIJ;
nc{'FermiLevel'}(:)  = EFERMI; 
nc{'PlaneWaveCutoff'}(:) = ENMAX;

% nc{'ROTATION'} = 'ROTATION'; %
% nc{'row'} = 'row'; %
% nc{''} = ''; %
% nc{'ROTATION'}={'row','col','NROTATION'}; %
%  %
%  %
% nc{'ROTATION'}(:)=IGROP; %

nc = close(nc);

% nc = netcdf('vaspdata.nc', 'nowrite');  %
% description = nc.description(:)                      % Global attribute. %
% variables = var(nc);                                 % Get variable data. %
% for i = 1:length(variables) %
%    disp([name(variables{i}) ' =']), disp(' ') %
%   disp(variables{i}(:)) %
%    pause %
% end %
% nc = close(nc);                                      % Close the file. %
