% LoadPsp.m %
% 1. Read from PspList.txt the pseudopotential file list    %
% 2. Convert them to netcdf format using binary2netcdf utility %
%    (http://listserv.fysik.dtu.dk/pipermail/campos/2003-August/000633.html) %
% 3. Load netcdf files %


function [Psp, iflag_success] = LoadPsp(Psp, Input, iFlagBigendian);
  
  iflag_success = 1;
  
  fprintf (1, '\n');
  fprintf (1, '[LoadPsp]  Load DACAPO ultrasoft pseudopotential\n');
  elementsymbol;
  TypeMax = Psp{1}.TypeMax;
  
  tmpdatadir = Input.DFTDataDir;
  addorbital = Input.ConvertPsp.AddOrDeleteOrbital;
  if (addorbital==1),
    addorbital_L = Input.ConvertPsp.AddOrbital_L;
  elseif (addorbital==-1),
    deleteorbital = Input.ConvertPsp.DeleteOrbitalIndex;
  end;
  
  
  nPsp = round(length(Input.PseudopotList)/2);
  
  for ipsp = 1 : nPsp,
    PspSymbolList{ipsp} = Input.PseudopotList{2*ipsp-1};
    psppath{ipsp}       = Input.PseudopotList{2*ipsp};
  end;
  
  
  nTypeMax = length(PspSymbolList);
  if (nTypeMax < TypeMax),
    fprintf (1, '[LoadPsp]  Number of pseudopotentials is not corret.\n');
    iflag_success = 0;
    return;
  end;
  
  for type = 1 : TypeMax,
    tmpsymbol = [];
    if (length(Psp{type}.Symbol)>1),
      if (~isletter(Psp{type}.Symbol(2))),
        tmpsymbol = Psp{type}.Symbol(1);
      else,
        tmpsymbol = Psp{type}.Symbol(1:2);
      end;
    else
      tmpsymbol = Psp{type}.Symbol(1);
    end;
    
    iflag = 1;
    jtype = 1;
    jflag = 0;
    while ( ~jflag && (jtype<=nTypeMax) ),
      jflag = strcmp(tmpsymbol, PspSymbolList{jtype});
      jtype = jtype + 1;
    end;
    
    iflag = iflag*jflag;
    if (jflag),
      PspIndex(type) = jtype-1;
    else
      PspIndex(type) = 0;
    end;
  end;
  
  if (~iflag),
    fprintf(1, ['[LoadPsp]  Can not find corresponding pseudopotentials.\n']);
    iflag_success = 0;
    return;
  end;
  
  strb2n = which('binary2netcdf.x');
  if isempty(strb2n),
    fprintf(1, ['[LoadPsp]  Can not find "binary2netcdf.x".\n']);
    iflag_success = 0;
    return;
  end;


  
  for type = 1 : TypeMax,
    fprintf (1, '%s uses %s,\n', Psp{type}.Symbol, psppath{ PspIndex(type)});
    
    fn_PspNetCDF = [tmpdatadir PspSymbolList{PspIndex(type)} '.nc'];
    fn_PspTxt = [tmpdatadir PspSymbolList{PspIndex(type)} '.txt'];
    
    system(['setenv F_UFMTENDIAN big;  ' strb2n ' ' ...
            psppath{PspIndex(type)} ' ' fn_PspNetCDF ' > ' fn_PspTxt]);
    fprintf (1, 'which is converted to NetCDF file %s,\n', fn_PspNetCDF);
    fprintf (1, 'with brief summary in %s.\n', fn_PspTxt);
    PspNetCDF = netcdf(fn_PspNetCDF, 'nowrite');
    fprintf(1, '\n');
    
    Psp{type}.AtomicNumber = strmatch(PspSymbolList{PspIndex(type)},...
                                      ElementSymbol,'exact');
    
    Psp{type}.RadialGrid   = PspNetCDF{'RadialGrid'}(:)';
    Psp{type}.mesh         = length(Psp{type}.RadialGrid);
    Psp{type}.RadialGridAB = PspNetCDF{'RadialGridDiff'}(:)';  
    
    Psp{type}.ngrid = 2*(floor(Psp{type}.mesh/2)+1)-1;
    Psp{type}.rgriddiff = Psp{type}.RadialGridAB(1,1:Psp{type}.ngrid);
    
    Psp{type}.phiType = PspNetCDF{'OrbitalType'}(:);
    Psp{type}.phiMax  = length(Psp{type}.phiType);
    Psp{type}.phi_L   = floor( mod(Psp{type}.phiType, 100) / 10 );
    Psp{type}.rphi    = PspNetCDF{'Orbital_RadialFunction'}(:);
    
  
    % add additional orbitals %
    if (addorbital == 1),
      for iLL = 1 : length(addorbital_L),
        Psp{type}.phi_L = [Psp{type}.phi_L' addorbital_L(iLL) ];
        aa    = []; 
        tmpa  = [];
        aa    = Psp{type}.rphi;
        tmpa  = [aa' aa(1,:)'];
        Psp{type}.rphi   = [];
        Psp{type}.rphi   = tmpa';
        Psp{type}.phiMax = Psp{type}.phiMax + 1;
      end;
      
    elseif (addorbital == -1),
      Psp{type}.newphi_L = [];
      Psp{type}.newrphi  = [];
      for iLL = 1 : length(deleteorbital),
        for iphi = 1: length(Psp{type}.phi_L),
          if (iphi == deleteorbital(iLL)),
            Psp{type}.phiMax = Psp{type}.phiMax - 1;
          else
            tmpL = [];
            aa   = [];
            tmpa = [];
            tmpL = [ Psp{type}.newphi_L' Psp{type}.phi_L(iphi)];
            Psp{type}.newphi_L = tmpL';
            aa    = Psp{type}.rphi(iphi,:);
            tmpa  = [Psp{type}.newrphi' aa'];
            Psp{type}.newrphi  = [];
            Psp{type}.newrphi  = tmpa';
          end;
        end;
      end;
      
      Psp{type}.phi_L = [];
      Psp{type}.rphi  = [];
      Psp{type}.phi_L = Psp{type}.newphi_L;
      Psp{type}.rphi  = Psp{type}.newrphi;
    end;
    
    
    iorb = 0;
    for iphi = 1 : Psp{type}.phiMax,
      for M = -Psp{type}.phi_L(iphi) : Psp{type}.phi_L(iphi),
        iorb = iorb + 1;
        Psp{type}.iphi(iorb) = iphi;
        Psp{type}.orb_L(iorb) = Psp{type}.phi_L(iphi);
        Psp{type}.orb_M(iorb) = M;
      end;
    end;
    
    Psp{type}.orbMax = iorb;
    

    
    fprintf(1, 'L (quantum number):   ');
    for iLL = 1 : length(Psp{type}.orb_L),
      fprintf(1, '   %d', Psp{type}.orb_L(iLL) );
    end;
    fprintf(1, '\n');
    
    close(PspNetCDF);
    fprintf (1, '\n');
  end;
  
  
  
  
