% LoadVASP.m %

function [Atom, Psp, PW, Bloch] = LoadVASP(dft_outfile_nc)
  
  fprintf (1, '\n');
  fprintf (1, '[LoadVASP]  Load VASP pseudopotentials\n');
  
 
  ResultNetCDF = netcdf(dft_outfile_nc, 'nowrite');
  TypeMax      = size(ResultNetCDF{'AtomSymbol'},1);
  
  for itype = 1:TypeMax,
    if ( strcmp(ResultNetCDF{'AtomSymbol'}(itype,2), ' ')),
      Psp{itype}.Symbol = ResultNetCDF{'AtomSymbol'}(itype, 1);
    else
      Psp{itype}.Symbol = ResultNetCDF{'AtomSymbol'}(itype, 1:2);
    end;
    Psp{itype}.TypeMax = TypeMax;
    
    Psp{itype}.Q = squeeze(ResultNetCDF{'AugmentationQij'}(itype, :, :));
    Psp{itype}.projMax = rank(Psp{itype}.Q);
    
    % add %
    tmpbetaL = ResultNetCDF{'ReciprocalProjectorK_L'}(itype, :);
    indup = max(find(tmpbetaL<10));
    Psp{itype}.beta_L = tmpbetaL(1:indup);
    Psp{itype}.beta_Gmax = ResultNetCDF{'ReciprocalProjectorKGMax'}(itype);    
    
  end;
  
  number_IBZ_kpoints = size(ResultNetCDF('number_IBZ_kpoints'),1);

  for type = 1 : TypeMax,
    iproj = 0;
    Psp{type}.betaMax = length(Psp{type}.beta_L);
    for ibeta = 1 : Psp{type}.betaMax,
      for M = -Psp{type}.beta_L(ibeta) : Psp{type}.beta_L(ibeta),
        iproj = iproj + 1;
        Psp{type}.ibeta(iproj)  = ibeta;
        Psp{type}.proj_L(iproj) = Psp{type}.beta_L(ibeta);
        Psp{type}.proj_M(iproj) = M;
      end;
    end;
    Psp{type}.projMax = iproj;
  end;
  
  betak = ResultNetCDF{'ReciprocalProjectorK'};  
  for itype = 1 : TypeMax,
    for ibeta = 1 : Psp{itype}.betaMax,
      Psp{itype}.rbetaK(ibeta, :) = betak(itype,ibeta ,:);
    end;
    Psp{itype}.rbetaK_LENGTH = size(Psp{itype}.rbetaK,2);
  end;  
  
%   for ibz = 1: number_IBZ_kpoints, %
%     for itype = 1 : TypeMax, %
%       for iproj = 1 : Psp{itype}.projMax, %
%         Psp{itype}.projK_IBZ(iproj, :, ibz) = ResultNetCDF{'ReciprocalProjector'}(itype, iproj, ibz, :); %
%       end; %
%     end; %
%   end; %
  
  %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%  
  
  fprintf (1, '\n');
  fprintf (1, '[LoadVASP]  Load VASP atomic information\n');

  Atom.TypeMax = TypeMax;
  DynamicAtomSpecies = ResultNetCDF{'DynamicAtomSpecies'}(:);
  Atom.AtomMax = size(DynamicAtomSpecies,1);
  for iatom = 1 : Atom.AtomMax,
    itype = DynamicAtomSpecies(iatom, 2);
    Atom.AtomSymbol{iatom} = Psp{itype}.Symbol;
    Atom.Atom2Type(iatom)  = itype;
  end;
  
  for itype = 1 : Atom.TypeMax,
    Atom.Type2Atom{itype} = find(Atom.Atom2Type == itype);
  end;
  
  UnitCell = ResultNetCDF{'UnitCell'};
  Atom.NumberIonicSteps = size(UnitCell,1);
  Atom.H = UnitCell(Atom.NumberIonicSteps,:,:); 
  Atom.Volume = abs(det(Atom.H));
  
  Atom.G = 2 * pi * inv(Atom.H);
  
  DynamicAtomPositions = ResultNetCDF{'DynamicAtomPositions'};
  for iatom = 1 : Atom.AtomMax,
    Atom.S(iatom,:) = DynamicAtomPositions(Atom.NumberIonicSteps,iatom,:);
    Atom.X(iatom,:) = Atom.S(iatom,:) * Atom.H;
  end;
  


  %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%  

  
  
  fprintf (1, '\n');
  fprintf (1, '[LoadVASP]  Load VASP Bloch wavefunction\n');
  
  number_IBZ_kpoints = size(ResultNetCDF('number_IBZ_kpoints'),1);
  number_plane_waves = size(ResultNetCDF('number_plane_waves'),1);
  
  NumberIonicSteps = size(UnitCell,1);
  PW.Volume   = abs(det(Atom.H));
  G           = Atom.G;
  IBZK        = ResultNetCDF{'IBZKpoints'}(:);
  
  PW.IBZK     = IBZK;
  PW.IBZKMax  = size(PW.IBZK,1);
  
  IBZKWeight     = ResultNetCDF{'KpointWeight'}(:);
  PW.IBZKWeight  = IBZKWeight/sum(IBZKWeight);
  KpointSetup    = ResultNetCDF{'KpointSetup'}(:);
  PW.KpointSetup = KpointSetup';
  BZKMax         = KpointSetup(1) * KpointSetup(2) * KpointSetup(3);
  PW.BZKMax      = BZKMax;
  [BZK]          = Generate_BZK(PW.KpointSetup);
  PW.BZK         = BZK;
  
  tmpWaveFunction = ResultNetCDF{'WaveFunction'};
  tmpWave         = ResultNetCDF{'WaveFunction'}(:);
  PW.nIBZKs = size(tmpWaveFunction,1);
  PW.nSpins = size(tmpWaveFunction,2);
  PW.nBands = size(tmpWaveFunction,3);
  PW.nWaves = size(tmpWaveFunction,4);
  
  WaveFunction = reshape(tmpWave, ...
                         PW.nIBZKs, PW.nSpins, PW.nBands, PW.nWaves, 2);
  tmpWaveFunction = [];
  
  tmpEigenValues  = ResultNetCDF{'EigenValues'};
  tmpEigen        = ResultNetCDF{'EigenValues'}(:);
  PW.nIonicSteps  = size(tmpEigenValues,1);
  EigenValues     = reshape(tmpEigenValues(PW.nIonicSteps,:,:,:), PW.nIBZKs, PW.nSpins, PW.nBands);
  
  PW.EigenValues  = EigenValues;
  
  Bloch = cell(PW.IBZKMax, PW.nSpins, PW.nBands);
  
  for ibz = 1 : PW.IBZKMax,
    for ispin = 1 : PW.nSpins,
      for iband = 1 : PW.nBands,
        Bloch{ibz,ispin,iband}.a = ...
            ( WaveFunction(ibz,ispin,iband,:,1) + ...
              j * WaveFunction(ibz,ispin,iband,:,2) ) / sqrt(PW.Volume);
        Bloch{ibz,ispin,iband}.a = Bloch{ibz,ispin,iband}.a(:);
      end;
    end;
  end;
  
  tmpOccupation   = ResultNetCDF{'OccupationNumbers'};
  PW.Occupation   = reshape(tmpOccupation(PW.nIonicSteps,:,:,:), PW.nIBZKs, PW.nSpins, PW.nBands);
  
  % for VASP only. (in VASP, for ispin = 1, occup = 1) %
  %    (Note that in DACAPO, for ispin = 1, occup = 2) %
  if (round(PW.nSpins)==1),
    PW.Occupation = PW.Occupation*2;
  end;
  %%%%%%%%%%%%%%%%%%  
  
  EnergyCutOff    = ResultNetCDF{'PlaneWaveCutoff'}(:);
  PW.EnergyCutOff = EnergyCutOff(:);
  Efermi    = ResultNetCDF{'FermiLevel'}(:);
  PW.Efermi = Efermi;
  
  FFTIndex = ResultNetCDF{'WaveFunctionFFTindex'}(:);
  NGX = size(ResultNetCDF('softgrid_dim1') ,1);
  NGY = size(ResultNetCDF('softgrid_dim2') ,1);
  NGZ = size(ResultNetCDF('softgrid_dim3') ,1);
  PW.nGrid = [NGX NGY NGZ];
  NG = PW.nGrid;
  
  for ibz = 1 : PW.IBZKMax,
    f=FFTIndex(ibz,:);
    x= mod(f,NGX );
    indx = find(x==0);
    x(indx) = NGX;
    y= mod(f-x, NGY*NGX)/NGX +1;
    z= (f-x-(y-1)*NGX)/(NGY*NGX) +1;
    indf = [x' y' z'];
    PW.FFTIndex(ibz,:,:) = indf';
  end;
  PW.IBZGmin = ResultNetCDF{'NumberPlaneWavesKpoint'}(:);
  
  
  PW.oldIBZK    = PW.IBZK;
  PW.oldIBZKMax = PW.IBZKMax;
  PW.oldIBZKWeight = PW.IBZKWeight;

  PW.oldIBZK  = PW.oldIBZK * G';  
  PW.IBZK     = PW.IBZK * G';
  PW.BZK      = PW.BZK * G';
  
  
  % ROTATE WAVEFUNCTION in IBZ TO GET WAVEFUNCTION IN FULL BZ %
  nLatRot = size(ResultNetCDF{'LatticeRotationMatrix'}, 1);
  LatRot = ResultNetCDF{'LatticeRotationMatrix'}(:);
  if (PW.BZKMax> round(2*PW.IBZKMax-1) ),
    numspa = nLatRot;
    
    for ispa = 1: numspa,

      %%%%%%%%%%%%%%%%%%%%%%%%      
      % Here  for VASP, it uses tranpose of LatRot, due to collumn-wise output %
      %  but for DACAPO, no tranpose is needed for LatRot %
      LatRot(ispa,:,:)  = squeeze(LatRot(ispa,:,:))';
      CartRot(ispa,:,:) = G * squeeze(LatRot(ispa,:,:)) * inv(G);
    end;
    PW.LatRot  = LatRot;
    PW.CartRot = CartRot;
    
    [PW, Bloch, Psp] = QuickWavefunction(PW, Bloch, Atom.G, Atom.H, Psp); 
   
  end;
  
  
  close(ResultNetCDF);
  
  
  return;
  
