% QuickDOS.m %

function [QMB] = QuickDOS(Input, QMB)
  
  fprintf (1, '[QuickDOS]  Calculate density of states\n');
  fprintf (1, '\n');
  
  TBHamOverlap    = QMB.TBHamOverlap;
  TBSOverlap      = QMB.TBSOverlap;
  Rdim            = QMB.Rdim;
  Enk             = QMB.Enk_PW;
  number_of_spin  = QMB.nSpins;
  atomMax         = QMB.atomMax;
  atomtype        = QMB.atomtype;
  G               = QMB.G;
  H               = QMB.H;
  Efermi          = QMB.Efermi;
  orbMax          = QMB.orbMax;
  atom_X          = QMB.atom_X;
  
  oldIBZKWeight   = QMB.oldIBZKWeight;
  oldIBZK         = QMB.oldIBZK;
  
  GenEigThreshold = Input.GenEig.Threshold;
  saveDir  = Input.saveDir;
  
  fnameeps = [saveDir '/Quambo.DOS.eps'];
  fname    = Input.DOS.Filename;
  broadeningwidth = Input.DOS.BroadeningWidth;
  Emin     = Input.DOS.EnergyRange(1);
  Emax     = Input.DOS.EnergyRange(2);
  npoints  = Input.DOS.EnergyGridPoints;
  Epoint   = [0:(npoints-1)] * (Emax - Emin)/ (npoints-1) + Emin + Efermi;
  
  %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
  
  
  Hdim = 0;
  for iatom = 1:atomMax,
    itype = atomtype(iatom);
    Hdim  = Hdim + orbMax(itype);
  end;
  
  id = 0;
  for ixx = -Rdim(1) : Rdim(1),
    ix = ixx + Rdim(1) + 1;
    for iyy = -Rdim(2) : Rdim(2),
      iy = iyy + Rdim(2) + 1;
      for izz = -Rdim(3) : Rdim(3),
        iz = izz + Rdim(3) + 1;
        
        id = id + 1;
        RCellID(id, :)  = [ix iy iz];
        RCellPos(id, :) = [ixx iyy izz];
        
      end;
    end;
  end;
  
  EnDOS   = zeros(length(Epoint), number_of_spin);
  tmpdos  = [];
  Enktemp = [];
  for ispin = 1: number_of_spin,
    for ikpt = 1 : length(oldIBZKWeight),
      %       Kvector = G * rand(3,1); %
      Kvector = oldIBZK(ikpt, :)';
      
      HH = zeros(Hdim, Hdim);
      SS = zeros(Hdim, Hdim);
      
      OrbX  = 0;
      OrbXX = 0;
      for iatom = 1 : atomMax,
        itype = atomtype(iatom);
        for iorb = 1 : orbMax(itype),
          OrbXX = OrbX + iorb;
          OrbY = 0;
          OrbYY = 0;
          for jatom = 1 : atomMax,
            jtype = atomtype(jatom);
            for jorb = 1 : orbMax(jtype),
              OrbYY = OrbY + jorb;
              
              HH(OrbXX, OrbYY) = ...
                  HH(OrbXX, OrbYY) + ...
                  sum( squeeze(TBHamOverlap(ispin, OrbXX, OrbYY, :)) .* ...
                       exp( i *( RCellPos(:,:) * H ) * Kvector) );
              
              SS(OrbXX, OrbYY) = ...
                  SS(OrbXX, OrbYY) + ...
                  sum( squeeze(TBSOverlap(ispin, OrbXX, OrbYY, :)) .*  ...
                       exp( i *( RCellPos(:,:) * H ) * Kvector) );
              
            end;
            OrbY = OrbY + orbMax(jtype);
          end;
          
        end;
        OrbX = OrbX + orbMax(itype);
      end;
      
      dd = GenEig(HH,SS, GenEigThreshold);
      %     [vv, dd] = eig(HH, SS); %
      Enktemp = [Enktemp dd'];
      for ig = 1 : length(dd),
        tmpdos = - ((Epoint - real(dd(ig)))/broadeningwidth).^2;
        EnDOS(:,ispin)  = EnDOS(:,ispin) + oldIBZKWeight(ikpt) * exp(tmpdos)';
      end;
      
    end;
  end;
  
  EnDOS = EnDOS/sqrt(pi)/broadeningwidth;
  
  hfig =figure('visible', 'off');
  plot(Epoint-Efermi, EnDOS(:,1),'r')
  if (number_of_spin==2),
    hold on;
    plot(Epoint-Efermi, EnDOS(:,2),'b')
    legend('majority spin','minority spin')
    xlabel('Energy (eV)');
    ylabel('arbitary unit');
  end;
  
  fnameeps = [saveDir '/Quambo.DOS.eps'];
  print('-depsc', '-r300', fnameeps);
  close(hfig);
  
  DOSAll.atomMax  = atomMax;
  DOSAll.EnDOS    = EnDOS;
  DOSAll.Epoint   = Epoint;
  DOSAll.Efermi   = Efermi;
  DOSAll.number_of_spin = number_of_spin;
  
  QMB.DOS = DOSAll;
  
  fprintf(1, '\n');
  
  
