% QuickPDOS.m %

function [QMB] = QuickPDOS(Input, QMB)
  
  fprintf (1, '[QuickPDOS] Calculate projected density of states\n');
  fprintf (1, '\n');

  TBHamOverlap    = QMB.TBHamOverlap;
  TBSOverlap      = QMB.TBSOverlap;
  Rdim            = QMB.Rdim;
  Enk             = QMB.Enk_PW;
  number_of_spin  = QMB.nSpins;
  atomMax         = QMB.atomMax;
  atomtype        = QMB.atomtype;
  G               = QMB.G;
  H               = QMB.H;
  Efermi          = QMB.Efermi;
  orbMax          = QMB.orbMax;
  atom_X          = QMB.atom_X;
  newIBZK         = QMB.IBZK;
  newIBZKWeight   = QMB.IBZKWeight;
  oldIBZKWeight   = QMB.oldIBZKWeight;
  oldIBZK         = QMB.oldIBZK;
  GenEigThreshold = Input.GenEig.Threshold;
  saveDir         = Input.saveDir;

  
  fnameeps        = [saveDir '/Quambo.PDOS.eps'];
  fname           = Input.PDOS.Filename;
  broadeningwidth = Input.DOS.BroadeningWidth;
  Emin            = Input.DOS.EnergyRange(1);
  Emax            = Input.DOS.EnergyRange(2);
  npoints         = Input.DOS.EnergyGridPoints;
  Epoint          = [0:(npoints-1)] * (Emax - Emin)/ (npoints-1) + Emin + Efermi;
  
  %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
  
  
  Hdim = 0;
  for iatom = 1:atomMax,
    itype = atomtype(iatom);
    Hdim  = Hdim + orbMax(itype);
  end;
  
  id = 0;
  for ixx = -Rdim(1) : Rdim(1),
    ix = ixx + Rdim(1) + 1;
    for iyy = -Rdim(2) : Rdim(2),
      iy = iyy + Rdim(2) + 1;
      for izz = -Rdim(3) : Rdim(3),
        iz = izz + Rdim(3) + 1;
        
        id = id + 1;
        RCellID(id, :)  = [ix iy iz];
        RCellPos(id, :) = [ixx iyy izz];
        
      end;
    end;
  end;
  
  EnPDOS   = zeros(length(Epoint), number_of_spin, Hdim);
  tmppdos  = [];
  

  
  for ispin = 1: number_of_spin,
    for ikpt = 1 : size(newIBZK,1),
      
      Kvector = newIBZK(ikpt, :)';
      
      HH = zeros(Hdim, Hdim);
      SS = zeros(Hdim, Hdim);
      
      OrbX  = 0;
      OrbXX = 0;
      for iatom = 1 : atomMax,
        itype = atomtype(iatom);
        for iorb = 1 : orbMax(itype),
          OrbXX = OrbX + iorb;
          OrbY = 0;
          OrbYY = 0;
          for jatom = 1 : atomMax,
            jtype = atomtype(jatom);
            for jorb = 1 : orbMax(jtype),
              OrbYY = OrbY + jorb;
              
              HH(OrbXX, OrbYY) = ...
                  HH(OrbXX, OrbYY) + ...
                  sum( squeeze(TBHamOverlap(ispin, OrbXX, OrbYY, :)) .* ...
                       exp( i *( RCellPos(:,:) * H ) * Kvector) );
              
              SS(OrbXX, OrbYY) = ...
                  SS(OrbXX, OrbYY) + ...
                  sum( squeeze(TBSOverlap(ispin, OrbXX, OrbYY, :)) .*  ...
                       exp( i *( RCellPos(:,:) * H ) * Kvector) );
              
            end;
            OrbY = OrbY + orbMax(jtype);
          end;
          
        end;
        OrbX = OrbX + orbMax(itype);
      end;
      
      [vv, dd] = GenEigVD(HH, SS, GenEigThreshold);  
      
      %     [vv, dd] = eig(HH, SS);  %
      %     dd = diag(dd); %
      
      for ig = 1 : length(dd),
        tmppdos = - ((Epoint - real(dd(ig)))/broadeningwidth).^2;
        for kdim = 1 : Hdim,
          EnPDOS(:,ispin, kdim)  = EnPDOS(:,ispin, kdim) + ...
              abs(vv(kdim, ig))/norm(vv(:,ig)) * newIBZKWeight(ikpt) * exp(tmppdos)';
        end;
      end;
      
      
    end;
  end;
  
  EnPDOS = EnPDOS/sqrt(pi)/broadeningwidth;
  
  
  
  PDOSAll.atomMax  = atomMax;
  PDOSAll.EnPDOS   = EnPDOS;
  PDOSAll.Epoint   = Epoint;
  PDOSAll.Efermi   = Efermi;
  PDOSAll.number_of_spin = number_of_spin;
  
  QMB.PDOS = PDOSAll;
  
  save(fname,'PDOSAll', 'Input');
  fprintf(1,['projected density of states data file save as:  ' fname '\n']);
  fprintf(1, '\n');
  
  
