% QuickTBBandKPath.m %

function [TBBandStructure] = QuickTBBandKPath(Input)
  
  Structname = Input.BandStructure.CrystalStructure;
  TBBandStructure.Name       = [];
  TBBandStructure.KpathLabel = [];
  TBBandStructure.Kpath      = [];

if (Structname=='DEF'),
  
  TBBandStructure.Name       = Input.BandStructure.CrystalStructure;
  TBBandStructure.KpathLabel = Input.BandStructure.DefinedKPathName;
  Kpath = zeros( 3, length(TBBandStructure.KpathLabel));
  for ikpath = 1 : length(TBBandStructure.KpathLabel),
    Kpath(:,ikpath) = Input.BandStructure.DefinedKPath(ikpath,:)';
  end;
  TBBandStructure.Kpath  = Kpath;
  
  
elseif (Structname=='FCC'),

%%%%%%%%%%%% FCC K point %%%%%%%%%%%%
% // FCC_Gamma = [ 0    0    0   ]'; %
% // FCC_X     = [ 0    0    1   ]'; %
% // FCC_L     = [ 1/2  1/2  1/2 ]'; %
% // FCC_W     = [ 1/2  0    1   ]'; %
% // FCC_U     = [ 1/4  1/4  1   ]'; %
% // FCC_K     = [ 3/4  0    3/4 ]'; %
% // FCC_XP    = [ 1    0    1   ]'; %
% GFCC = [ -0.5 0.5 0.5; ... %
%          0.5 -0.5 0.5; ... %
%          0.5 0.5 -0.5; ]; %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

FCC_Gamma = [ 0    0    0   ]';
FCC_X     = [ 0    1/2  1/2 ]';
FCC_L     = [ 1/2  1/2  1/2 ]';
FCC_W     = [ 1/4  3/4  1/2 ]';
FCC_U     = [ 1/4  5/8  5/8 ]';
FCC_K     = [ 3/8  3/4  3/8 ]';
FCC_XP    = [ 1/2  1    1/2 ]';

Kpath(:,1) = FCC_Gamma;
Kpath(:,2) = FCC_X;
Kpath(:,3) = FCC_W;
Kpath(:,4) = FCC_L;
Kpath(:,5) = FCC_Gamma;
Kpath(:,6) = FCC_K;
Kpath(:,7) = FCC_XP;
TBBandStructure.Name   = ['FCC'];
TBBandStructure.Kpath  = Kpath;
TBBandStructure.KpathLabel = ['GXWLGKX'];

%  %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

elseif (Structname == 'BCC'),

    
%%%%%%%%%%%% BCC K point %%%%%%%%%%%%
% // BCC_Gamma = [ 0    0    0   ]'; %
% // BCC_H     = [ 1    0    0   ]'; %
% // BCC_N     = [ 1/2  1/2  0   ]'; %
% // BCC_P     = [ 1/2  1/2  1/2 ]'; %
% BCC_New = inv(GBCC) * BCC_OLD /2 %
% GBCC = [ 0   0.5 0.5; ... %
%          0.5 0   0.5; ... %
%          0.5 0.5 0; ]; %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

BCC_Gamma = [ 0    0    0   ]';
BCC_H     = [ -1/2  1/2   1/2  ]';
BCC_N     = [ 0   0   1/2   ]';
BCC_P     = [ 1/4  1/4  1/4 ]';

Kpath(:,1) = BCC_Gamma;
Kpath(:,2) = BCC_H;
Kpath(:,3) = BCC_N;
Kpath(:,4) = BCC_P;
Kpath(:,5) = BCC_Gamma;
Kpath(:,6) = BCC_N;
TBBandStructure.Name   = ['BCC'];
TBBandStructure.Kpath  = Kpath;
TBBandStructure.KpathLabel = ['GHNPGN'];

% Kpath(:,1) = BCC_Gamma; %
% Kpath(:,2) = BCC_H; %
% Kpath(:,3) = BCC_P; %
% Kpath(:,4) = BCC_N; %
% Kpath(:,5) = BCC_Gamma; %
% Kpath(:,6) = BCC_P; %
% TBBandStructure.Name   = ['BCC']; %
% TBBandStructure.Kpath  = Kpath; %
% TBBandStructure.KpathLabel = ['GHPNGP']; %

elseif (Structname == 'BC2'),

    
%%%%%%%%%%%% BCC K point %%%%%%%%%%%%
% // BCC_Gamma = [ 0    0    0   ]'; %
% // BCC_H     = [ 1    0    0   ]'; %
% // BCC_N     = [ 1/2  1/2  0   ]'; %
% // BCC_P     = [ 1/2  1/2  1/2 ]'; %
% BCC_New = inv(GBCC) * BCC_OLD /2 %
% GBCC = [ 0   0.5 0.5; ... %
%          0.5 0   0.5; ... %
%          0.5 0.5 0; ]; %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

BCC_Gamma = [ 0    0    0   ]';
BCC_H     = [ -1/2  1/2   1/2  ]';
BCC_N     = [ 0   0   1/2   ]';
BCC_P     = [ 1/4  1/4  1/4 ]';

Kpath(:,1) = BCC_Gamma;
Kpath(:,2) = BCC_H;
Kpath(:,3) = BCC_N;
Kpath(:,4) = BCC_Gamma;
Kpath(:,5) = BCC_P;
Kpath(:,6) = BCC_N;
Kpath(:,7) = BCC_P;
Kpath(:,8) = BCC_H;
TBBandStructure.Name   = ['BCC'];
TBBandStructure.Kpath  = Kpath;
TBBandStructure.KpathLabel = ['GHNGPNPH'];


elseif (Structname == 'SCC'),

    
%%%%%%%%%%%% Simple Cubic K point %%%%%%%%%%%%
% SCC_Gamma = [ 0    0    0    ]'; %
% SCC_X     = [ 0    1/2  0    ]'; %
% SCC_M     = [ 1/2  1/2  0    ]'; %
% SCC_R     = [ 1/2  1/2  1/2  ]'; %
% SCC_New = inv(GSCC) * SCC_OLD /2 %
% GSCC = [ 0   0.5 0.5; ... %
%          0.5 0   0.5; ... %
%          0.5 0.5 0; ]; %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

SCC_Gamma = [ 0    0    0    ]';
SCC_X     = [ 0    1/2  0    ]';
SCC_M     = [ 1/2  1/2  0    ]';
SCC_R     = [ 1/2  1/2  1/2  ]';
SCC_A     = [ 0    0    1/2  ]';
SCC_Z     = [ 1/2  0    1/2  ]';


Kpath(:,1) = SCC_Gamma;
Kpath(:,2) = SCC_M;
Kpath(:,3) = SCC_X;
Kpath(:,4) = SCC_Gamma;
Kpath(:,5) = SCC_R;
Kpath(:,6) = SCC_X;
Kpath(:,7) = SCC_Z;
Kpath(:,8) = SCC_A;
% Kpath(:,9) = SCC_Gamma; %
% Kpath(:,10) = SCC_Z; %
% Kpath(:,11) = SCC_R; %

TBBandStructure.Name   = ['SCC'];
TBBandStructure.Kpath  = Kpath;
TBBandStructure.KpathLabel = ['GMXGRXZA'];
% TBBandStructure.KpathLabel = ['GMXGRXZAGZR']; %

elseif (Structname=='HCP'),

%%%%%%%%%%%%%%%%%%%%%%%% HCP K point %%%%%%%%%%%%
% // HCP_Gamma = [ 0    0    0   ]'; %
% // HCP_X     = [ 0    0    1   ]'; %
% // HCP_L     = [ 1/2  1/2  1/2 ]'; %
% // HCP_W     = [ 1/2  0    1   ]'; %
% // HCP_U     = [ 1/4  1/4  1   ]'; %
% // HCP_K     = [ 3/4  0    3/4 ]'; %
% // HCP_XP    = [ 1    0    1   ]'; %
% primitive lattice vectors %
%   a1 = ( 1/2 a , - \sqrt(3)/2 a , 0 ) %
%   a2 = ( 1/2 a , + \sqrt(3)/2 a , 0 ) %
%   a3 = ( 0 ,       0 ,            c ) %
% GHCP = [ 2*\pi/a, -2*\pi / sqrt(3)a,  o ; ... %
%          2*\pi/a, +2*\pi / sqrt(3)a,  o ; ... %
%          0,        0,      c; ];              %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

HCP_Gamma = [ 0    0    0   ]';
HCP_M     = [ 0    1/2  0   ]';
HCP_L     = [ 0    1/2  1/2 ]';
HCP_A     = [ 0    0    1/2 ]';
HCP_H     = [ 1/3  1/3  1/2 ]';
HCP_S     = [ 1/6  1/6  1/2 ]';
HCP_T     = [ 1/6  1/6  0   ]';
HCP_K     = [ 1/3  1/3  0   ]';

Kpath(:,1) = HCP_Gamma;
Kpath(:,2) = HCP_M;
Kpath(:,3) = HCP_L;
Kpath(:,4) = HCP_A;
Kpath(:,5) = HCP_Gamma;
Kpath(:,6) = HCP_K;
Kpath(:,7) = HCP_H;
Kpath(:,8) = HCP_A;

TBBandStructure.Name   = ['HCP'];
TBBandStructure.Kpath  = Kpath;
TBBandStructure.KpathLabel = ['GMLAGKHA'];

%  %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

elseif (Structname == 'CHA'),

%%%%%%%%%%%%%%% 1D Chain %%%%%%%%%%%%%%%
Chain_X1     = [  0    0   -1/2]';
Chain_Gamma  = [ 0    0    0   ]';
Chain_X2     = [  0    0   1/2 ]';

% Chain_X1     = [  -1/2  0    0   ]'; %
% Chain_Gamma  = [ 0    0      0   ]'; %
% Chain_X2     = [  1/2  0    0    ]'; %
Kpath(:,1) = Chain_Gamma;
Kpath(:,2) = Chain_X1;
% Kpath(:,3) = Chain_X2; %
TBBandStructure.Name   = ['1D Chain'];
TBBandStructure.Kpath  = Kpath;
TBBandStructure.KpathLabel = ['GX'];
% TBBandStructure.KpathLabel = ['XGX']; %


% Chain_Gamma     = [  0    0    0    ]'; %
% ChainImag_Gamma = [  0    0   1*i   ]'; %
% Chain_X         = [  0    0   1/2   ]'; %
% ChainImag_X     = [  0    0   i+1/2 ]'; %

% Chain_Gamma     = [  0    0    0    ]'; %
% ChainImag_Gamma = [  1*i    0    0  ]'; %
% Chain_X         = [ 1/2    0    0     ]'; %
% ChainImag_X     = [  i+1/2  0    0    ]'; %

% Kpath(:,1) = ChainImag_Gamma; %
% Kpath(:,2) = Chain_Gamma; %
% Kpath(:,3) = Chain_X; %
% Kpath(:,4) = ChainImag_X; %
% TBBandStructure.Name   = ['1D Chain']; %
% TBBandStructure.Kpath  = Kpath; %
% TBBandStructure.KpathLabel = ['IGXI']; %

else
  
  fprintf(1,['[QuickTBBandKPath]   No structure of K path sampling']);
  fprintf (1, '\n');
  return;
  
end;
%  %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

return;
