% QuickTBBandStructure.m %

function [ QMB ] = QuickTBBandStructure(Input,  QMB)
  
  fprintf (1, '[QuickTBBandStructure]  Calculate tightbinding band structure\n');
  fprintf (1, '\n');
  
  TBHamOverlap    = QMB.TBHamOverlap;
  TBSOverlap      = QMB.TBSOverlap;
  Rdim            = QMB.Rdim;
  Enk             = QMB.Enk_PW;
  number_of_spin  = QMB.nSpins;
  atomMax         = QMB.atomMax;
  atomtype        = QMB.atomtype;
  G               = QMB.G;
  H               = QMB.H;
  Efermi          = QMB.Efermi;
  orbMax          = QMB.orbMax;
  atom_X          = QMB.atom_X;
  
  
  
  saveDir         = Input.saveDir;
  GenEigThreshold = Input.GenEig.Threshold;
  NKgrid     = Input.BandStructure.Grid;
  Lrcutoff   = Input.BandStructure.RCutoffYesOrNo;
  Rcutoff    = Input.BandStructure.RCutoff;
  Structname = Input.BandStructure.CrystalStructure;
  fname      = Input.BandStructure.Filename;
  
  clear Kpath;
  TBBandStructure =[];
  clear TBBandStructure;
  
  Hdim = size(TBHamOverlap,2);
  
  
  %%%%%%%%%%%%     Setup K-Sampling-Path    %%%%%%%%%%%%
  [TBBandStructure] = QuickTBBandKPath(Input);
  
  %%%%%%%%%%%%   BandStructure data storage %%%%%%%%%%%%
  TBBandStructure.NKgrid = NKgrid;
  TBBandStructure.Energy = ...
      1000 *ones( number_of_spin, ...
                  Hdim, ...
                  (size(TBBandStructure.Kpath,2)-1), ...
                  TBBandStructure.NKgrid );
  
  TBBandStructure.EnergyNew = ...
      1000 *ones( number_of_spin, ...
                  Hdim, ...
                  (size(TBBandStructure.Kpath,2)-1)*TBBandStructure.NKgrid );
  
  ipt = 1;
  KptStart = TBBandStructure.Kpath(:,1);
  for ipath = 2: (length(TBBandStructure.KpathLabel)),
    kx = (TBBandStructure.Kpath(1,ipath) - KptStart(1))/TBBandStructure.NKgrid;
    ky = (TBBandStructure.Kpath(2,ipath) - KptStart(2))/TBBandStructure.NKgrid;
    kz = (TBBandStructure.Kpath(3,ipath) - KptStart(3))/TBBandStructure.NKgrid;
    
    for ikgrid = 1: TBBandStructure.NKgrid,
      TBBandStructure.KGrid(:,ipt) = ...
          [kx ky kz]'*(ikgrid-1) + KptStart;
      prepoint  = TBBandStructure.KGrid(:,ipt);
      ipt = ipt+1;
    end;
    KptStart = TBBandStructure.Kpath(:,ipath);
  end;
  
  prelen = 0.;
  nkptlen = size(TBBandStructure.KGrid,2);
  TBBandStructure.KLength = zeros(nkptlen, 1);
  for ipt = 2 : nkptlen,
    TBBandStructure.KLength(ipt) = ...
        TBBandStructure.KLength(ipt-1) + ...
        norm(TBBandStructure.KGrid(:,ipt) - TBBandStructure.KGrid(:,ipt-1));
  end;
  
  
  TBBandStructure.Efermi = Efermi;
  %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
  
  Kpath = G * TBBandStructure.Kpath;
  
  
  fprintf(1,['Calculating ' TBBandStructure.Name ' band structure\n']);
  fprintf (1, '\n');
  
  Hdim = 0;
  for iatom = 1:atomMax,
    itype = atomtype(iatom);
    Hdim  = Hdim + orbMax(itype);
  end;
  
  RCellID  = [];
  RCellPos = [];
  id = 0;
  for ixx = -Rdim(1) : Rdim(1),
    ix = ixx + Rdim(1) + 1;
    for iyy = -Rdim(2) : Rdim(2),
      iy = iyy + Rdim(2) + 1;
      for izz = -Rdim(3) : Rdim(3),
        iz = izz + Rdim(3) + 1;
        
        id = id + 1;
        RCellID(id, :)  = [ix iy iz];
        RCellPos(id, :) = [ixx iyy izz];
        
      end;
    end;
  end;
  
  Distance = [];
  for iatom = 1: atomMax,  
    itype = atomtype(iatom);
    for jatom = 1: atomMax,
      jtype = atomtype(jatom);
      for id = 1 : size(RCellID,1),
        Distance(iatom, jatom, id) = ...
            norm(RCellPos(id,:) * H + atom_X(jatom, :) - atom_X(iatom,:) );
      end;
    end;
  end;
  
  
  RCellIDTemp  = [];
  RCellPosTemp = [];
  if (Lrcutoff ~= 0),
    for iatom = 1 : atomMax,
      Ratomi = atom_X(iatom,:);
      for jatom = 1 : atomMax,
        idx = 1;
        for id = 1 : size(RCellID,1),
          Ratomj   = RCellPos(id, :)*H + atom_X(jatom,:);
          DRatomij = abs(norm(Ratomj - Ratomi));
          if (DRatomij <= Rcutoff),
            RCellIDTemp(iatom, jatom, idx) = id;
            RCellPosTemp(iatom, jatom, idx, 1:3) = RCellPos(id, :);
            idx = idx + 1;
          end;
        end; % for id = 1 : size(RCellID,1)
        RCellIDMax(iatom,jatom) = idx-1;
      end;
    end;
  end;
  
  mkpt = 1;
  for ispin = 1 : number_of_spin,
    fprintf(1,'ispin = %d', ispin);
    ikpt = 1;
    
    fprintf(1,'    iKpath = ');
    for iKpath = 1:(size(Kpath,2)-1),
      
      fprintf(1,'%d ',iKpath);
      
      deltaK = [Kpath(:,iKpath+1) - Kpath(:,iKpath)] / NKgrid;
      
      for iKpoint = 1 : NKgrid, 
        
        Kvector = Kpath(:,iKpath) + deltaK * (iKpoint - 1);
        if (Lrcutoff == 0),
          matPhase = exp( i *( RCellPos(:,:) * H ) * Kvector);
        end;
        
        HH = zeros(Hdim, Hdim);
        SS = zeros(Hdim, Hdim);
        
        OrbX  = 0;
        OrbXX = 0;
        
        
        for iatom = 1 : atomMax,
          itype = atomtype(iatom);
          Ratomi = atom_X(iatom,:);
          
          
          for iorb = 1 : orbMax(itype),
            OrbXX = OrbX + iorb;
            
            OrbY = 0;
            OrbYY = 0;
            for jatom = 1 : atomMax,
              jtype = atomtype(jatom);
              
              if (Lrcutoff ~= 0),
                tempPOS = reshape(RCellPosTemp(iatom, jatom, 1:RCellIDMax(iatom,jatom) , 1:3), ...
                                  RCellIDMax(iatom,jatom), 3);
                matPhaseCutoff = ...
                    exp( i *( tempPOS * H ) * Kvector);
                matIDCutoff   = ...
                    squeeze(RCellIDTemp(iatom, jatom, 1:RCellIDMax(iatom,jatom)));
              end;
              
              for jorb = 1 : orbMax(jtype),
                OrbYY = OrbY + jorb;
                
                if (Lrcutoff == 0),
                  HH(OrbXX, OrbYY) = ...
                      HH(OrbXX, OrbYY) + ...
                      sum( squeeze(TBHamOverlap(ispin, OrbXX, OrbYY, :)) .* ...
                           matPhase );
                  
                  SS(OrbXX, OrbYY) = ...
                      SS(OrbXX, OrbYY) + ...
                      sum( squeeze(TBSOverlap(ispin, OrbXX, OrbYY, :)) .*  ...
                           matPhase );
                else
                  HH(OrbXX, OrbYY) = ...
                      HH(OrbXX, OrbYY) + ...
                      sum( ...
                          squeeze(TBHamOverlap(ispin, OrbXX, OrbYY, matIDCutoff )) .* ...
                          matPhaseCutoff);
                  SS(OrbXX, OrbYY) = ...
                      SS(OrbXX, OrbYY) + ...
                      sum( ...
                          squeeze(TBSOverlap(ispin, OrbXX, OrbYY, matIDCutoff)) .*  ...
                          matPhaseCutoff );
                end;   % if (Lrcutoff == 0)
                
              end;
              OrbY = OrbY + orbMax(jtype);
            end;
            
          end;
          OrbX = OrbX + orbMax(itype);
        end;
        
        dd = [];
        dd = GenEig(HH, SS, GenEigThreshold);
        
        [vv2, dd2] = eig(HH, SS);
        TBBandStructure.EigenVector(:,:,mkpt) = vv2;
        mkpt = mkpt + 1;
        
        TBBandStructure.Energy(ispin, 1:length(dd), iKpath, iKpoint) = ...
            sort(real(dd))';
        
        TBBandStructure.EnergyNew(ispin, 1:length(dd), ikpt ) = sort(real(dd))';
        ikpt = ikpt + 1;
        
      end;
      
    end;
    fprintf (1, '\n');
    
  end;
  
  
  
  BS = zeros(number_of_spin, Hdim+1, (size(Kpath,2)-1)* NKgrid );
  basicvalue = 0;
  for ispin = 1 : number_of_spin,
    for iKpath = 1 : (size(Kpath,2)-1),
      ds = norm(Kpath(:,iKpath+1) - Kpath(:,iKpath));
      BS(ispin, 1, ((iKpath-1)*NKgrid+1) : (iKpath*NKgrid) ) = ...
          basicvalue + [1:NKgrid]*ds;
      basicvalue = BS(ispin,1,iKpath*NKgrid);
      
      for iKpoint = 1 : NKgrid,
        BS(ispin, 2:(Hdim+1), ((iKpath-1)*NKgrid+1) : (iKpath*NKgrid) ) = ...
            squeeze(TBBandStructure.Energy(ispin, 1:Hdim, iKpath, 1:NKgrid));
      end;
      
    end;
  end;
  
  TBBandStructure.BandData = BS;
  
  save(fname,'TBBandStructure', 'Input', 'TBHamOverlap', 'TBSOverlap');
  fprintf(1,['\ndata file save as:\n' fname '\n']);
  fprintf(1, '\n');
  
  QMB.TBBandStructure = TBBandStructure;
  
  iflag_PlotBandStructure = QuickPlotTBBandStructure(Input, QMB);
  
  return;
  
