% main.m %
% Xiaofeng Qian, MIT (Jul. 26, 2005) %

clear all;
close all;

% Load physical constants %
% units; %

% Required: Load input file of parameters %
QuamboInput;

if ( ~exist('Input', 'var') ),
  fprintf(1, '\nQuambo Input is not available. \n stopped.\n\n');
  break;
end;

if (~exist(Input.saveDir,'dir')),
  system(['mkdir ' Input.saveDir]);
end;


dft_outfile_base = 'DFTResult';
dft_outfile_nc   = [ dft_outfile_base '.nc' ];
dft_outfile_txt  = [ dft_outfile_base '.txt' ];

system(['ln -sf ' Input.DFTNetCDF ' ' dft_outfile_nc]);
system(['ls -l ' dft_outfile_nc]);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
if (Input.LoadOldQuamboDataOrNOT ~= 1),
    

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%  
  if (strcmp(Input.DFTType, 'DACAPO')),
    
    
    %   Load DACAPO PSP data %
    iFlagBigendian = 1;
    [Psp, iFlag_success]    = LoadPspDACAPO(Input, iFlagBigendian);
    if (~iFlag_success),
      fprintf(1,['\n[main]  failed at LoadPspDACAPO. \n'])
      break;
    end;
    
    %   Convert DACAPO PSP data %
    ralpha   = Input.ConvertPsp.OrbitalScaling;
    plotflag = Input.ConvertPsp.ShowScaledOrbital;
    add_del_orb  = Input.ConvertPsp.AddOrDeleteOrbital;
    tmpsavedir   = Input.saveDir;
    
    [ Psp ] = ConvertPspDACAPO(Psp, ralpha, plotflag, add_del_orb, tmpsavedir);

    %   Load DACAPO DFT data %
    [Atom, PW, Bloch] = LoadDACAPO(dft_outfile_nc, Psp);
    
    [ Psp, Atom ] = ...
        QuickPspDACAPO(Psp, Atom, PW.FFTIndex, PW.nWaves, ...
                     PW.nGrid, PW.IBZKMax, PW.IBZK, PW.IBZGmin);
    
    
    
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
  elseif (strcmp(Input.DFTType, 'VASP')),
    
    %   Convert VASP DFT data into NetCDF format   %
    if (~exist(Input.DFTNetCDF, 'file')),
      vasp2net;
    end;
    
    %   Load VASP DFT data %
    [Atom, Psp, PW, Bloch] = LoadVASP(dft_outfile_nc);
    
    %   Load DACAPO PSP data %
    iFlagBigendian = 1;
    [Psp, iFlag_success]    = LoadPspVASP(Psp,  Input, iFlagBigendian);
    if (~iFlag_success),
      fprintf(1,['\n[main]  failed at LoadPspVASP. \n'])
      break;
    end;
    
    
    ralpha   = Input.ConvertPsp.OrbitalScaling;
    plotflag = Input.ConvertPsp.ShowScaledOrbital;
    add_del_orb = Input.ConvertPsp.AddOrDeleteOrbital;
    tmpsavedir   = Input.saveDir;
    
    %   Convert DACAPO PSP data %
    [ Psp ] = ConvertPspVASP(Psp, ralpha, plotflag, add_del_orb, tmpsavedir);
    [ Psp, Atom ] = ...
        QuickPspVASP(Psp, Atom, PW.FFTIndex, PW.nWaves, ...
                     PW.nGrid, PW.IBZKMax, PW.IBZK, PW.IBZGmin);
        
  else
    
    fprintf(1,'\nNo Input.DFTType available');
    break;
  end;
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


  % Required: Quambo procedure to optimally select virtual unoccupied space %
  Eshift  = Input.CombineUnoccupiedBV.EFermiShift;
  condmax = Input.CombineUnoccupiedBV.MaxConditionNumber;
  ntotalband = Input.CombineUnoccupiedBV.TotalPlaneWaveBands;
  [QMB, PsiCoeff, BlochBasis, PW, bsb, bb] = ...
      CombineUnoccupiedBVSpin(Eshift, condmax, ntotalband, Atom, Psp, Bloch, PW);
  
else
  fprintf(1, ['\nLoading Old QUAMBO Data: ' Input.QUAMBOData.Filename '\n']);
  load(Input.QUAMBOData.Filename);
end;
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%




%%%%%%%%%%%%%%%%%%%%%%%%% Optional Procedures %%%%%%%%%%%%%%%%%%%%%%%%%%
% Optional: Calculate tight binding matrix based on QUAMBOs %
if (Input.Tightbinding.CalculateOrNot==1),
  [ QMB ] = QuickTBOverlap(Input.Tightbinding.DataFilename, ...
                           Atom, PsiCoeff, Psp, PW, BlochBasis, QMB);
end;

% Optional: Using "TBHamOverlap" and "TBSOverlap" to calculate bandstructure %
if (Input.BandStructure.CalculateOrNot==1),
  [ QMB ] = QuickTBBandStructure(Input, QMB);
end;

% Optional: Using "TBHamOverlap" and "TBSOverlap" to calculate density of states %
if (Input.DOS.CalculateOrNot==1),
  [ QMB ] = QuickDOS(Input, QMB);
  [ QMB ] = QuickPDOS(Input, QMB);
end;

% Optional: Using "TBHamOverlap" and "TBSOverlap" to calculate 3D Fermi surface %
if (Input.Fermisurface.CalculateOrNot==1),
  [ QMB ] = QuickFermiSurface(Input, QMB);
end;

% Optional: calculate Mulliken overlap matrix element %
if (Input.Mulliken.CalculateOrNot==1),
  [QMB] = QuickMullikenCharge(Input, QMB, PsiCoeff, PW.Occupation);
end;

% Optional: 1. Generate QUAMBO in real space and ... %
%           2. store data in matlab format for Bond.m calculation %
if (Input.OutputQuambo.CalculateOrNot),
  [iflag_QMB2MAT] = QMB2MAT(Input, Atom, Psp, PW, BlochBasis);
end;

if (Input.SaveFinalQUAMBODataOrNot),
  save(Input.SaveFinalQUAMBO.Filename, 'QMB', 'Input');
end;

save(Input.Tightbinding.DataFilename,'QMB',  'PsiCoeff', 'BlochBasis', 'Atom', 'PW', 'Psp');


%   TEST CODES %
%   CheckWavefunction; %
%   [EnkQuambo, EnormdiffTB] = CheckTBOverlap(QMB, Input); %
