% QuickCutPlane.m %

fprintf(1,'QuickCutPlane: Initializing\n');

LBruteForce = 1;
ispin = 1;
iatom = 1;
iorb = 4;
type = atomtype(iatom);

deltax = 1/8;

% Al %
% 100 %
% [y, z] = meshgrid(-4:deltax:4, -4:deltax:4); %
% x = atom_X(iatom,2) *ones(size(z,1),size(z,2));  %
% phi = z * 0; %

% Si %
% 110 %
lentag = [-6:deltax:6];
d0     = 0;
[x, z] = meshgrid(lentag, atom_X(iatom,3) + lentag) ;
y = d0-x;
phi = x * 0;



WaveAmp = zeros(IBZKMax, size(BlochBasis{1,ispin,1}.a,2));

for ibz = 1 : IBZKMax,
  for ispin = 1 : number_of_spin,
    for iband = 1 : number_of_bands,
      WaveAmp(ibz,:) = WaveAmp(ibz,:) + ...
          BlochBasis{ibz,ispin,iband}.orbS(iatom,iorb) * ...
          BlochBasis{ibz,ispin,iband}.a;
      end;
  end;
end;


if (LBruteForce == 1),
  
  fprintf(1,'QuickCutPlane: Generating cut-plane\n');
  fprintf(1,'ibz ');
  
  for ibz = 1 : IBZKMax,
    GList = zeros(length(WaveFunctionFFTindex(1,1,:)), 3);
    GList = GListGenerator(ibz);
    GList = GList * G';
    
    tmpa = find(sum(GList.^2,2)==0);
    tmpb = find(tmpa>1);
    Gmin = tmpa(min(tmpb))-1;
    GLength = length(GList);
    if isempty(Gmin),
      Gmin = GLength;
    end;

    for iwave = 1 : Gmin,
      
      K = GList(iwave,:) + IBZK(ibz,:);
      
      phase = K(1)*x + K(2)*y + K(3)*z;
      phi = phi + ...
            IBZKWeight(1,ibz) * real (WaveAmp(ibz,iwave) * exp( j * phase ) );
%        %
%       if (norm(IBZK(ibz,:))~=0),  %
%         phi = phi + 2 * real (WaveAmp(ibz,iwave) * exp( j * phase ) ); %
%       else  %
%         phi = phi + WaveAmp(ibz,iwave) * exp( j * phase ); %
%       end;  %
      
    end;
    fprintf(1,'%d ',ibz);
  end;
  
% 100 & fcc %
  figure;
  contour(y,z,real(phi),32);
  axis equal;
  
  hold on;
  grid on;
  t = abs(H(1,3));
  axis([-2*t 2*t -2*t 2*t])
  line([-t t ], [-t t],'LineWidth',4,'Color',[.8 .8 .8])
  line([-t t ], [t -t],'LineWidth',4,'Color',[.8 .8 .8])
  
  line([-t*2 0], [0 -2*t],'LineWidth',4,'Color',[.8 .8 .8])
  line([ t*2 0], [0  2*t],'LineWidth',4,'Color',[.8 .8 .8])  
  line([0 -t*2], [ 2*t 0],'LineWidth',4,'Color',[.8 .8 .8])
  line([0  t*2], [-2*t 0],'LineWidth',4,'Color',[.8 .8 .8])
  
% 110 & diamond %
%   figure; %
%   contour(x*sqrt(2),z,real(phi),13); %
%   axis equal; %
%    %
%   hold on; %
%   grid on; %
%   t = abs(H(1,3)); %
%   st = sqrt(2); %
%   axis([-2*t 2*t -2*t 2*t]) %
%   t = abs(atom_X(2,1)); %
%   st = sqrt(2); %
%   line([-st*t 0 ], [-t 0],'LineWidth',4,'Color',[.8 .8 .8]) %
%   line([ st*t 0 ], [-t 0],'LineWidth',4,'Color',[.8 .8 .8]) %
%   line([-st*t -2*st*t], [-t 0],'LineWidth',4,'Color',[.8 .8 .8]) %
%   line([ st*t 2*st*t], [-t 0],'LineWidth',4,'Color',[.8 .8 .8])   %
  
else
  
  phi = x *0.;
  
  for ibz = 1 : IBZKMax,
    [GIndex, ingx, ingy, ingz] = GIndexGenerator(ibz,0,0); 
    GIndex = GIndex .*(GIndex>0);
    Gmin = round(min(find(GIndex(:)==0)-1));
    if isempty(Gmin)
      Gmin=length(GIndex);
    end;
    
    GQuamboReshaped = zeros(1,ingx*ingy*ingz);
    GQuamboReshaped(1, GIndex(1:Gmin,1)) = WaveAmp(ibz,1:Gmin);
    GQuamboPlaneReshaped = reshape(GQuamboReshaped, ingx,ingy,ingz);
    QuamboCube  = ifftn(GQuamboPlaneReshaped);
    QuamboPlane = interp(x,y,z, QuamboCube);
    K = GList + IBZK(ibz,:);
    phase = K(1)*x + K(2)*y + K(3)*z;
    
    phi = phi + IBZKWeight(1,ibz) * real(QuamboPlane .* exp( j * phase ) );
    
  end;
  
  contour(x,z,real(phi),20);
end;
