% QuickSBloch.m %
% calculate S_unitcell |Bloch_{nk}>  in fourier space %
% Scaling: ProjK / sqrt(volume) %
%          Bloch * sqrt(volume) %
% QuickProjBloch( iatom,jproj, ibz,ispin,iband ) %
%     =  ProjBloch(iatom,jproj, ibz,ispin,iband) %

function SBloch = QuickSBloch(ibz, wave, Psp, atomtype, atomMax, volume, atom_phase);
  
  number_plane_waves = length(wave);
  
  SBloch = zeros( 1, number_plane_waves );
  twave = wave(:);
  wave = [];
  wave = twave.';
  
  SBloch(1, :) = wave(1,:) * sqrt(volume);
  
  for iatom = 1 : atomMax,
    type = atomtype(iatom);
    
    for iproj = 1:Psp{type}.projMax,
      for jproj = 1:Psp{type}.projMax,
        if ( abs(Psp{type}.Q(iproj, jproj)) > 0. ),
          
          %  Q |G+k> <G+k|beta_n(G+k)> < beta_m(G+k) | Bloch_{nk}> * exp( -i*(G+k)*R_i ) %
          tmpwave = QuickProjBloch(atom_phase(iatom, :, ibz), ...
                                   Psp{type}.projK(jproj, :, ibz), ...
                                   wave, ...
                                   volume);
          
          ss= Psp{type}.Q( iproj,jproj ) .* ...
              tmpwave * ...
              conj( Psp{type}.projK(iproj,:,ibz) ) .* ...
              conj( atom_phase(iatom,:, ibz) );
          
          SBloch(1,:)= SBloch(1, :) + ss(1,:);
          
        end;
        
      end;
      
    end;
    
  end;
  
  return;
  
  
  
  
  
  
