!----------------------------------------------------------------------------
!
program binary2netcdf
use netcdfinterface
!
!     converts pseudopotential files from fortran binary form to netcdf 
!     form
!
!     this version is compatible only with pseudopotential program
!     version 3.0.0 or higher.  it assumes logarithmic mesh.
!
!=======================================================================
implicit none 

integer, parameter :: idim1 = 1000,idim3=10,idim4=5,idim5=idim4,idim6=2*idim5-1 
integer, parameter :: idim8 = 20

!.....radial mesh information
      real*8  ::  r(idim1),rab(idim1)
!.....charge densities
      real*8, allocatable  ::  rsatom(:),rspsco(:)
!.....pseudo quantum numbers, energies occupancies and cutoff radii
      integer ::  nnlzps(idim4)
      real*8  ::  eeps(idim4),wwnlps(idim4),rc(idim5)
      real*8,allocatable  ::  wf(:,:)
!.....beta and q functions and q pseudization coefficients
      real*8,allocatable   :: beta(:,:),qfunc(:,:,:)
      real*8, allocatable  :: qfcoef(:,:,:,:)
      real*8 vloc(idim1),vloc0(idim1),rinner(idim6)
!.....indexing for the beta functions
      integer :: nbl0(idim5),nbl(idim5)
!.....angular momenta, reference energy, qij and dij of vanderbilt scheme
      integer :: lll(idim3)
      real*8  :: eee(idim3)
      integer :: iptype(idim3)
      real*8, allocatable  :: qqq(:,:)
      real*8, allocatable  :: ddd0(:,:),ddd(:,:) 
!.....logic to keep track of logarithmic vs herman skillman meshes
      logical :: tlog
!.....version numbers and date
      integer :: iver(3),idmy(3)
 
!.....title
      character*20 title,xctype
!.....file name array
      character*360 infil,outfil

      integer,parameter :: input=5,iout=6,iops = 14
      real*8            :: zps,zvps,exftps,etot,eloc,qtryc,ptryc,rcloc
      real*8            :: rpcor
      integer           :: mesh,nvalps,keyps,ifpcor,nang,lloc,nqf,irel,nlc
      integer           :: ifqopt,nbeta,kkbeta,npf,lmaxps,nchi

!.....define netcdf dimension names
      integer                   :: angularchannels
      character*(*) , parameter :: nvalps_name          = 'NumberValenceStates'
      character*(*) , parameter :: mesh_name            = 'RadialGridDimension'
      character*(*) , parameter :: nang_name            = 'NumberAngularMomentum'
      character*(*) , parameter :: angularchannels_name = 'NumberAngularChannels'
      character*(*) , parameter :: nbeta_name           = 'NumberBetaFunctions'
      character*(*) , parameter :: kkbeta_name          = 'RadialGridDimension_BetaFunctions'
      character*(*) , parameter :: nqf_name             = 'NumberAugmentationFunctionCoefficients'


      integer i,j,k,lp,ib,ifip2 

      integer status
      integer ncid
      integer  iargc
      external iargc
 
 
!     Get run time parameters
!     infil             (input file name)
!     outfil            (output file name)
!
!
      call getarg(1,infil    )
      call getarg(2,outfil)
      write(iout,*) 'name of input file ',infil
      write(iout,*) 'name of output file ', outfil

      write(iout,*) 'working on basis that file has version prefix'
 
!     r e a d  p s e u d o p o t e n t i a l  f r o m  f i l e
!
      open( unit = iops , file = infil , status = 'old' ,form = 'unformatted' )
!
      read (iops) (iver(i),i=1,3),(idmy(i),i=1,3)
!
!     specify logarithmic mesh
      tlog = .true.
!
      read (iops) title,zps,zvps,exftps,nvalps,mesh,etot
!
!     check that the array dimensions are sufficient
      if (mesh.gt.idim1) call error(' reform',' mesh.gt.idim1',mesh)
      if (nvalps.gt.idim4) call error(' reform',' nvalps.gt.idim4', nvalps)
 
      read (iops) (nnlzps(i),wwnlps(i),eeps(i),i=1,nvalps)


      read (iops) keyps,ifpcor,rinner(1)
 
      if ( iver(1) .ge. 3 ) then
         read(iops) nang,lloc,eloc,ifqopt,nqf,qtryc
      endif

      if (10*iver(1)+iver(2).ge.51) then
         read (iops) (rinner(i),i=1,nang*2-1)
      endif
 
      if ( iver(1) .ge. 4 ) then
         read(iops) irel
      endif
 
!     set the number of angular momentum terms in q_ij to read in
      if ( iver(1) .eq. 1 ) then
!         no distinction between nang and nvalps
          nang = nvalps
!         no optimisation of q_ij so 3 term taylor series
          nqf = 3
          nlc = 5
      elseif ( iver(1) .eq. 2 ) then
!         no distinction between nang and nvalps
          nang = nvalps
!         no optimisation of q_ij so 3 term taylor series
          nqf = 3
          nlc = 2 * nvalps - 1
      else
          nlc = 2 * nang - 1
      endif
 
      if (nang.gt.idim5) call error(' reform',' nang.gt.idim5',nang)
      if (nqf.gt.idim8) call error(' reform',' nqf.gt.idim8',nqf)

      read (iops) (rc(i),i=1,nang)
      read (iops) nbeta,kkbeta
 
      if (nbeta.gt.idim3) call error(' reform',' nbeta.gt.idim3',nbeta)

      allocate(beta(kkbeta,nbeta))
      allocate(ddd0(nbeta,nbeta),ddd(nbeta,nbeta))
      allocate(qfunc(kkbeta,nbeta,nbeta))
      allocate(qfcoef(nqf,nlc,nbeta,nbeta))
      allocate(qqq(nbeta,nbeta))
      do 100 j=1,nbeta
          read (iops) lll(j),eee(j),(beta(i,j),i=1,kkbeta)
          do 100 k=j,nbeta
            read (iops) ddd0(j,k),ddd(j,k),qqq(j,k),  & 
            (qfunc(i,j,k),i=1,kkbeta),                & 
            ((qfcoef(i,lp,j,k),i=1,nqf),lp=1,nlc)
            ddd0(k,j)=ddd0(j,k)
            ddd (k,j)=ddd (j,k)
            qqq (k,j)=qqq (j,k)
            do i=1,kkbeta
              qfunc(i,k,j)=qfunc(i,j,k)
            enddo  
      do 110 i=1,nqf
        do 110 lp = 1,nlc
          qfcoef(i,lp,k,j)=qfcoef(i,lp,j,k)
      110 continue
  100 continue
 
      if (10*iver(1)+iver(2).ge.72) then
          read (iops) (iptype(j),j=1,nbeta),npf,ptryc
      endif
 
      read (iops) rcloc,(vloc0(i),i=1,mesh)
      
!     set index arrays nbl and nbl0
      do 150 lp = 1,nang
          nbl(lp) = 0
  150 continue
      do 160 ib = 1,nbeta
          lp = lll(ib) + 1
          nbl(lp) = nbl(lp) + 1
  160 continue
      lmaxps = lll(nbeta)
      nbl0(1) = 0
      do 170 i = 1,lmaxps
          nbl0(i+1) = nbl0(i) + nbl(i)
  170 continue

!     possible readin of the frozen core correction
      rpcor = 0.
      allocate(rspsco(mesh))
      do 177 i = 1,mesh
          rspsco(i) = 0.
 177   continue
      
      if (ifpcor.gt.0) then
          read (iops) rpcor
          read (iops) (rspsco(i),i=1,mesh)
      endif
      
      read (iops) (vloc(i),i=1,mesh)
      allocate(rsatom(mesh))
      read (iops) (rsatom(i),i=1,mesh)
      
!     possible further read in of information in log mesh case
      if ( tlog ) then
          read (iops) (r(i),i=1,mesh)
          read (iops) (rab(i),i=1,mesh)
      endif
 
      if (iver(1) .ge. 6) then
!           nchi = nvales
           nchi = nvalps
           if (iver(1) .ge. 7)  read (iops) nchi
           if (nchi.ne.nvalps) then 
             write(*,*) 'Warning nchi not equal to nvalps'
           endif
           allocate(wf(mesh,nchi))
           read (iops) ((wf(i,j), i=1,mesh),j=1,nchi)
      endif
 
      close (iops)
 
      write (iout,190)
  190 format ('1pseudopotential has been read in')
!
!
!-----------------------------------------------------------------------
!
!              p s e u d o p o t e n t i a l  r e p o r t
!
      if (exftps.eq. 0.) xctype = '      ceperley-alder'
      if (exftps.eq.-1.) xctype = '              wigner'
      if (exftps.eq.-2.) xctype = '     hedin-lundqvist'
      if (exftps.eq.-3.) xctype = ' gunnarson-lundqvist'
      if (exftps.eq. 1.) xctype = ' C-A + B88gx + LYPgc'
      if (exftps.eq. 2.) xctype = ' C-A + B88gx        '
      if (exftps.eq. 3.) xctype = ' C-A + B88gx + P86gc'
      if (exftps.eq. 4.) xctype = ' Perdew Wang 1991   '
      if (exftps.eq. 5.) xctype = ' PBE - GGA          '
 
      write (iout,1000) (iver(i),i=1,3),idmy(2),idmy(1),idmy(3)
 1000 format (/4x,60(1h=)/4x,'|  pseudopotential report: version', & 
         i3,'.',i1,'.',i1,' date',i3,'-',i2,'-',i4,2x,'|',         & 
         /4x,60(1h-))
      write (iout,1010) title,xctype
 1010 format (4x,'|  ',2a20,' exchange-corr  |')
      write (iout,1020) zps,zvps,exftps
 1020 format (4x,'|  z =',f5.0,4x,'zv =',f5.0,4x,'exfact =',f10.5,13x,'|')
      write (iout,1030) etot
 1030 format (4x,'|     ',9x,'    ',9x,' etot  =',f10.5,13x,'|')
      write (iout,1040)
 1040 format(4x,'|  index    orbital      occupation    energy',14x,'|')
      write (iout,1050) (i,nnlzps(i),wwnlps(i),eeps(i),i=1,nvalps)
 1050 format(4x,'|',i5,i11,5x,f10.2,f12.2,15x,'|')
      if (10*iver(1)+iver(2).ge.51) then
        write (iout,1061) keyps,ifpcor,rpcor
 1061   format(4x,'|  keyps =',i2,5x,'ifpcor =',i2,5x,'rpcor =',f10.5,10x,'|')
        do 1064 i = 1,2*nang-1
          write (iout,1065) rinner(i),i
 1064   continue
 1065   format(4x,'|  rinner =',f10.2,5x,'for L=',i5,22x,'|')
      else
        write (iout,1060) keyps,ifpcor,rinner(1)
 1060   format(4x,'|  keyps =',i2,5x,'ifpcor =',i2,5x,'rinner =',f10.4,9x,'|')
      endif
 
      if (keyps.le.2) then
!
!       standard hsc or with vanderbilt smoothness
!       write (iout,1070)
!1070   format(4x,'|  hsc-type pseudo generation scheme:',22x,'|')
!       write (iout,1080) aa
!1080   format(4x,'|    aa = ',f8.3,41x,'|')
!       write (iout,1090) key
!1090   format(4x,'|    key =',3i8,25x,'|')
!       write (iout,1100) aaa
!1100   format(4x,'|    aaa =',3f8.3,25x,'|'
!    +   /4x,'|         l     rc',41x,'|')
!       write (iout,1110) (i,rc(i),i=1,nvalps)
!1110   format(4x,'|',4x,i6,f8.3,40x,'|')
!
      else if (keyps.eq.3) then
!
!       new scheme
        write (iout,1120)
 1120   format(4x,'|    new generation scheme:',32x,'|')
        write (iout,1130) nbeta,kkbeta,rcloc
 1130   format(4x,'|    nbeta = ',i2,5x,'kkbeta =',i5,5x,   & 
                       'rcloc =',f10.4,4x,'|'/              &
               4x,'|    ibeta      l    epsilon   rcut iptype',17x,'|')
        do 1140 ib=1,nbeta
          lp=lll(ib)+1
          if (10*iver(1)+iver(2).lt.72) iptype(ib)=-1
!           -1 means iptype was not recorded and is unknown
          write (iout,1150) ib,lll(ib),eee(ib),rc(lp),iptype(ib)
 1140   continue
 1150   format(4x,'|',6x,i2,5x,i2,4x,2f7.2,6x,i1,18x,'|')
 
        if (10*iver(1)+iver(2).ge.72) then
          ifip2=0
          do ib=1,nbeta
            if (iptype(ib).eq.2) ifip2=1
          end do
          if (ifip2.eq.1) then
            write (iout,1155) npf,ptryc
 1155       format(4x,'|  npf    =',i2,'  ptryc =',f8.3,29x,'|')
          endif
        endif
 
        if ( iver(1) .gt. 2 ) then
          write(iout,1160) lloc,eloc
 1160     format(4x,'|  lloc   =',i2,'  eloc   =',f8.3,28x,'|')
          write(iout,1170) ifqopt,nqf,qtryc
 1170     format(4x,'|  ifqopt =',i2,'  nqf    =',i2,'  qtryc =',f8.3, 17x,'|')
        endif
 
        if ( iver(1) .gt. 3 .and. irel .eq. 2 ) then
          write(iout,1180) 'koelling-harmon equation'
        else
          write(iout,1180) 'schroedinger equation   '
        endif
 1180   format(4x,'|',2x,'all electron calculation used ',a24,2x,'|')
 
        if ( .not. tlog ) then
          write(iout,2000) 'herman skillman mesh'
        else
          write(iout,2000) '**logarithmic mesh**'
        endif
 
 2000   format (4x,'|',9x,10('*'),a20,10('*'),9x,'|')
 
      else if (keyps.eq.4) then
!
!       frozen core
!       write (iout,2500)
!2500   format(4x,'|    frozen core all-electron case',25x,'|')
!
      endif
 
      write (iout,3000)
 3000 format (4x,60(1h=))


!     ----------------------------------------------------------------------------------------
!     write netcdf file

      angularchannels = 2*nang-1

      if (angularchannels.ne.nlc) call abort_calc(6,'angularchannels should be equal to nlc') 

!     open netcdf file
      status =  nf_create(outfil, NF_WRITE, ncid)
      if (status /= nf_noerr) call abort_calc(6, &
                "binary2netcdf -> nf_open : error opening nc-file")

!     write some neccesary dimensions
      status = nfputglobaldim(ncid,'dim3',3)
      status = nfputglobaldim(ncid,'dim20',20)

!     -----------------------------------------------------------------------------------------
!     global dimensions
      status = nfputglobaldim(ncid,nvalps_name         ,nvalps)
      status = nfputglobaldim(ncid,mesh_name           ,mesh)

      status = nfputglobaldim(ncid,nang_name           ,nang)
      status = nfputglobaldim(ncid,nbeta_name          ,nbeta)
      status = nfputglobaldim(ncid,kkbeta_name         ,kkbeta)
      status = nfputglobaldim(ncid,angularchannels_name,angularchannels)
      status = nfputglobaldim(ncid,nqf_name            ,nqf)
   

!     -----------------------------------------------------------------------------------------
!     define some introductory variables
      status = nfdefvar(ncid,'Title',NF_CHAR,dim_name1='dim20') 
      status = nfput(ncid,'Title',title)
      status = nfput(ncid,'ElementNumber',zps)
      status = nfput(ncid,'NumberValenceElectrons',zvps)

!     charge density 
      status = nfput(ncid,'ChargeDensity',rsatom(1:mesh),dim_name1=nvalps_name)

!     added by Xiaofeng Qian
!     frozen core correction
      status = nfput(ncid,'PartialCoreRadius', rpcor)
      status = nfput(ncid,'PartialCore',rspsco(1:mesh), dim_name1=mesh_name )
      
!     orbitals
      status = nfput(ncid,'OrbitalType',nnlzps(1:nvalps),      dim_name1=nvalps_name)
      status = nfput(ncid,'OrbitalOccupation',wwnlps(1:nvalps),dim_name1=nvalps_name)
      status = nfput(ncid,'OrbitalEnergy',eeps(1:nvalps),      dim_name1=nvalps_name)

      status = nfput(ncid,'Orbital_RadialFunction',wf(1:mesh,1:nvalps),dim_name1=mesh_name,dim_name2=nvalps_name)

!     -----------------------------------------------------------------------------------------
      
!     define beta projector functions 
      status = nfput(ncid,'BetaProjectors_RadialFunction',beta, dim_name1=kkbeta_name, & 
                                                dim_name2=nbeta_name ) 

      status = nfput(ncid,'BetaProjectors_RadialFunction%Description', & 
                          'r*Radial function for beta projector  Eq. 3 given on radial grid (RadialGrid)' )

      status = nfput(ncid,'BetaProjectors_AngularMomentum',lll(1:nbeta), dim_name1=nbeta_name ) 
      status = nfput(ncid,'BetaProjectors_AngularMomentum%Description','Angular momentum for each beta function') 

      status = nfput(ncid,'BetaProjectors_Energy',eee, dim_name1=nbeta_name ) 
      status = nfput(ncid,'BetaProjectors_Energy%Description','Energies at which  beta projector are constructed') 

!     -----------------------------------------------------------------------------------------
!     revised by Xiaofeng Qian, include Qnm_L(r) functions
!     define augmentation functions Qnm
      status = nfput(ncid,'AugmentationFunctions_Qnm',qfunc(1:kkbeta,1:nbeta,1:nbeta), & 
                     dim_name1 = kkbeta_name, dim_name2=nbeta_name, dim_name3=nbeta_name) 
      status = nfput(ncid,'AugmentationFunctions_Qnm%Description','Qnm_rad Eq. 26 defined outside R_inner radius') 

      status = nfput(ncid,'AugmentationFunctions_Rinner',rinner(1:angularchannels),dim_name1=angularchannels_name)
      status = nfput(ncid,'AugmentationFunctions_Rinner%Description','L-dependent inner cutoff radius') 
      status = nfput(ncid,'AugmentationFunctions_Rinner%Unit','Bohr') 

      status = nfput(ncid,'AugmentationFunctions_Coefficients',qfcoef(1:nqf,1:angularchannels,1:nbeta,1:nbeta), &
                dim_name1 = nqf_name,dim_name2=angularchannels_name,dim_name3=nbeta_name,dim_name4=nbeta_name)
      status = nfput(ncid,'AugmentationFunctions_Coefficients%Description', & 
                           'Expansions coefficients for the Qnm matrix, Eq. 30')

      
      status = nfput(ncid,'AugmentationFunctions_IntegratedQnm',qqq(1:nbeta,1:nbeta),  & 
                     dim_name1 = nbeta_name, dim_name2=nbeta_name )
      status = nfput(ncid,'AugmentationFunctions_IntegratedQnm%Description','qnm matrix, Eq. 6') 

!     -----------------------------------------------------------------------------------------
!     Dnm matrices 
      status = nfput(ncid,'Dnm_unscreened',ddd0, &
                     dim_name1 = nbeta_name, dim_name2=nbeta_name) 
      status = nfput(ncid,'Dnm_unscreened%Description','Dnm(0) matrix, Eq. 13') 

      status = nfput(ncid,'Dnm_screened',ddd, &
                     dim_name1 = nbeta_name, dim_name2=nbeta_name) 
      status = nfput(ncid,'Dnm_screened%Description','Dnm_I matrix, Eq. 13, atomic reference state')

!     -----------------------------------------------------------------------------------------
!     local potential: unscreened vloc0   &   screened vloc
!     added by Xiaofeng Qian : screened localpseudopotential vloc
      status = nfput(ncid,'LocalPseudoPotential_unscreened',vloc0, dim_name1 = mesh_name ) 
      status = nfput(ncid,'LocalPseudoPotential_unscreened%Description','local part of unscreened pseudo-potential')
      
      status = nfput(ncid,'LocalPseudoPotential_screened',vloc,  dim_name1 = mesh_name ) 
      status = nfput(ncid,'LocalPseudoPotential_screened%Description','local part of screened pseudo-potential')      
      
!     -----------------------------------------------------------------------------------------
!     local potential
!     old part
!     status = nfput(ncid,'LocalPseudoPotential',vloc0, dim_name1 = mesh_name ) 
!     status = nfput(ncid,'LocalPseudoPotential%Description','local part of pseudo-potential')

!     -----------------------------------------------------------------------------------------
!     radial grid 
      status = nfput(ncid,'RadialGrid',r, dim_name1 = mesh_name ) 
      status = nfput(ncid,'RadialGridDiff', rab, dim_name1 = mesh_name )

!     -----------------------------------------------------------------------------------------

!     close netcdf file 
      status = nf_close(ncid)
 
      end
 
!-----------------------------------------------------------------------
      subroutine error(a,b,n)
!-----------------------------------------------------------------------
      character*(*) a,b
      write(6,1) a,b,n
    1 format(//' program ',a,':',a,'.',8x,i8,8x,'stop')
      stop
      end
 
!========================================================
      subroutine abort_calc(iunit, message)
! -------------------------------------------------------
!    invoke clexit, after writing message
! -------------------------------------------------------
      implicit none
      integer         iunit
      character*(*)   message
 
      write(iunit,*) "abort_calc: ", trim(adjustl(message))
 
      end subroutine abort_calc 

