% loadvasp.m %

load WAVE
load EIGE
load IKPT
load FFTCOE
load FFTIND
load POSE
load QION
load BETAK
load BETAKGMAX
load BETA
load SYMM
[SPE] = textread('SPEC','%s');


NPSPNL = 100;

NKPT = FFTCOE(8,2); %Number of K-point
NB = FFTCOE(8,3);   %Number of band
H = zeros(3,3); %Unit cell in real cell
B = zeros(3,3); %Unit cell in reciprocal cell
H(1,:)=FFTCOE(2,:);
H(2,:)=FFTCOE(3,:);
H(3,:)=FFTCOE(4,:);
B(1,:)=FFTCOE(5,:);
B(2,:)=FFTCOE(6,:);
B(3,:)=FFTCOE(7,:);
NGX = FFTCOE(1,1);  %Number of G point along 3 vetors
NGY = FFTCOE(1,2); 
NGZ = FFTCOE(1,3);
EFERMI = EIGE(2,1);  %Fermi Energy
ENMAX = EIGE(2,2);  %Cutoff Energy
NPL =zeros(NKPT,1);    %Number of plane wave on each k-point
MAXNPL = 0;
ISPIN = EIGE(1,3);
for m = 1:NKPT
  NPL(m,1)= EIGE(2+m,1);
end
MAXNPL = max(NPL);
EIG = zeros(NKPT,ISPIN,NB);    %Eigenvalue on each k-point and band
FERWEI = zeros(NKPT,ISPIN,NB); %Fermi weight on each k-point and band


%Load information of Eigenvalues and Occupation number
for m =1:NKPT
  for l =1:ISPIN
    for n = 1:NB
    EIG(m,l,n) = EIGE(NKPT+2+(m-1)*NB+n+(l-1)*NKPT*NB,1);
    FERWEI(m,l,n) = EIGE(NKPT+2+(m-1)*NB+n+(l-1)*NKPT*NB,3);
    end
  end
end

 %Number of K point along 3 vetors in first BZ
NKX = IKPT(1,2);
NKY = IKPT(1,3);
NKZ = IKPT(1,4);
FULLKPT = int32(IKPT(1,2)*IKPT(1,3)*IKPT(1,4)); 
VKPT = zeros(NKPT,3);
WTKPT = zeros(NKPT,1);
for m = 1:NKPT
  VKPT(m,1) = IKPT(1+m,1);
  VKPT(m,2) = IKPT(1+m,2);
  VKPT(m,3) = IKPT(1+m,3);
  WTKPT(m,1) = IKPT(1+m,4);
end

%Atom Species, Number and POSITION information
NTYP = POSE(1,1);
NIONS = POSE(1,2);
SPEC = zeros(NIONS,2);
POS = zeros(NIONS,3);
for m = 1:NIONS
  POS(m,1) = POSE(1+m,2);
  POS(m,2) = POSE(1+m,3);
  POS(m,3) = POSE(1+m,4);
  SPEC(m,1)= m;
  SPEC(m,2)= int8(POSE(1+m,1));
end

%Augmentation Charge Qij
LMAX = zeros(NTYP,1);
MAXL = 0;
TotL = 0;
for m =1:NTYP
  LMAX(m,1) = QION(m,1);
  MAXL = max(LMAX(m,1),MAXL);
  TotL = TotL+LMAX(m,1);
end
LPS = zeros(NTYP,MAXL);
for m = 1:TotL
  n = int8(QION(m+NTYP,1)+0.1);
  l = int8(QION(m+NTYP,2)+0.1);
  LPS(n,l)=QION(m+NTYP,3);
end
MAXLM=0;
LMMAX =zeros(NTYP,1);
for m = 1:NTYP
  a =0;
  for n = 1:LMAX(m,1)
    a = a+2*LPS(m,n)+1;
  end
  LMMAX(m,1)=a;
  MAXLM = max(MAXLM,a);
end
n = length(QION(:,1));
qij = zeros(NTYP,MAXL,MAXL);
for m = NTYP+TotL:n;
  a = int8(QION(m,1)+0.1);
  b = int8(QION(m,2)+0.1);
  c = int8(QION(m,3)+0.1);
  qij(a,b,c)=QION(m,4);
  qij(a,c,b)=qij(a,b,c);
end
QIJ = zeros(NTYP,MAXLM,MAXLM);
A = zeros(MAXLM,MAXLM);
for NT = 1:NTYP
  for L =1:LMAX(NT)
    for LL =  L:LMAX(NT)
      if (LPS(NT,L) == LPS(NT,LL))
        maxlm =0;
        maxllm=0;
        for P = 1:L
          maxlm = maxlm+2*LPS(NT,P)+1;
        end
        for P = 1:LL
          maxllm =maxllm+ 2*LPS(NT,P)+1;
        end
      LMM = maxllm-2*LPS(NT,LL);
      for LM= maxlm-2*LPS(NT,L):maxlm
        QIJ(NT,LM,LMM)=qij(NT,L,LL);
        QIJ(NT,LMM,LM)=QIJ(NT,LM,LMM);
        LMM = LMM+1;
      end
      end   
    end
  end
end

%Rotation Matrix in K space
NROTK = SYMM(1,1);
IGROP = zeros(NROTK,3,3);
for m = 1:NROTK
  IGROP(m,1,:) = SYMM(3*m-1,:);
  IGROP(m,2,:) = SYMM(3*m,:);
  IGROP(m,3,:) = SYMM(3*m+1,:);
end

% 05/04/2006 %
QPROJK = zeros(NTYP,MAXL,NPSPNL);
PPP = size(BETAK,1);
for m = 1:PPP
  itype = round(BETAK(m,1));
  iL    = round(BETAK(m,2));
  QPROJK(itype, iL, 1:NPSPNL) = BETAK(m, 3:NPSPNL+2);
end

PROJ_L = 100*ones(NTYP,MAXL);
for itype = 1 : NTYP;
  for m = 1:LMAX(itype,1)
    PROJ_L(itype,1:LMAX) = LPS(itype, 1:LMAX);
  end
end;

%Porjector in G space fro each type of atom, at each lm orbit and k point
QPROJ=zeros(NTYP,MAXLM,NKPT,MAXNPL);
np =1;
for l = 1:NKPT
  for m = 1:NTYP
    for n = 1:LMMAX(m)
      for p = 1:NPL(l)
        QPROJ(m,n,l,p)=BETA(np);
        np = np+1;
      end
    end
  end
end

% Wavefunction FFT index
FFTINDEX = zeros(NKPT,MAXNPL);
l = 1;
for m = 1:NKPT
  for n = 1:NPL(m)    
    FFTINDEX(m,n) = FFTIND(l);
    l = l+1;
  end
end

% Wavefunction 
WAV = zeros(NKPT,ISPIN,NB,MAXNPL,2);
np = 1;
for l = 1:ISPIN
  for m = 1:NKPT
    for n = 1:NB
      for p = 1:NPL(m)
        WAV(m,l,n,p,1)=WAVE(np,1);
        WAV(m,l,n,p,2)=WAVE(np,2);
        np = np+1;
      end
    end
  end
end

% Atomic Symbol
spec = char(zeros(NTYP,2));
for m = 1:NTYP
  len = length(char(SPE(m)));
  if (len == 1),
    spec(m,1)=char(SPE(m));
  else
    a = char(SPE(m));
    spec(m,1)=a(1);
    spec(m,2)=a(2);
  end
end


