#include "symbol.inc"

!**********************SUBROUTINE NETCDF_OUT***********************************
!    Output all the necessary files in text form for matlab transformation
!    to netcdf dat, which can be used as Xiaofeng Qian's Quambo calculation.
!    
!                                         2006/01/11 Liang Qi
!***************************************************************************** 
      SUBROUTINE NETCDF_OUT(IRECLW,EFERMI,WDES,W,LATT_INI,IU0,NTYP,&
             KPOINTS,LATT_CUR,GRID,P,NONL_S,NROTK,NROT,IGRPOP)
      USE prec
      USE radial
      USE prec
      USE wave
      USE mgrid
      USE nonl
      USE lattice
      USE mkpoints
      USE pseudo 
      USE pot
      USE mpimy
      USE constant
      USE asa
      USE msymmetry
!     INCLUDE "pseudo.inc"

      
      IMPLICIT REAL(q) (A-H,O-Z)

      INTEGER IRECLW    ! record length
      TYPE (latt)     LATT_INI
      TYPE (latt)     LATT_CUR
      TYPE (wavedes)  WDES
      TYPE (wavedes1) WDES1
      TYPE (wavespin) W
      TYPE (grid_3d)     GRID
      TYPE (potcar) P(NTYP)
      TYPE (nonl_struct) NONL_S
      TYPE (kpoints_struct) KPOINTS

      
! local work arrays
      COMPLEX(q),ALLOCATABLE :: CW(:),EIG(:)
#ifdef WAVECAR_double
      COMPLEX(q), ALLOCATABLE :: CRD(:)
#else
      COMPLEX(qs),  ALLOCATABLE :: CRD(:)
#endif

      INTEGER   NROT,IGRPOP(3,3,48),NROTK      
      INTEGER :: LQNT,LQISPINOR,LQNK,LQLM,LQGN,LQLL,LQL,LQMMAX,LQLMIND
      INTEGER :: LQINDKPT, LQINDPW, LQNI, LQNPSNL
      INTEGER :: LQNKPX,LQNKPY,LQNKPZ,LQMAXK
      REAL(q) :: LQGXDX,LQGYDY,LQGZDZ
      COMPLEX(q) LQCGDR,LQCREXP
      
!  length of  betaprojector( |K|) 
      LQNPSNL = 100
      
      NPL_TOT = MAXVAL(WDES%NPLWKP_TOT)
      ALLOCATE(CW(NPL_TOT),CRD(NPL_TOT),EIG(WDES%NB_TOT))

      NODE_ME=0
      IONODE=0
#ifdef MPI
      NODE_ME=WDES%COMM%NODE_ME
      IONODE =WDES%COMM%IONODE
#endif
!
! header
!
      IF (IU0>=0) WRITE(IU0,*)'writing wavefunctions in text form'
      OPEN(1919,FILE='WAVE',STATUS='NEW')
     
      RISPIN=WDES%ISPIN
      RDUM  =IRECLW

#ifdef WAVECAR_double
      RTAG  =45210
#else
      RTAG  =45200
#endif


      IREC=2
      io_begin
      RNKPTS =WDES%NKPTS
      RNB_TOT=WDES%NB_TOT
      io_end
      OPEN(1920,FILE='EIGE',STATUS='NEW')
      WRITE(1920,*) WDES%NKPTS,WDES%NB_TOT,WDES%ISPIN
      WRITE(1920,*) EFERMI,WDES%ENMAX,0
      DO K=1,WDES%NKPTS
      WRITE(1920,*)WDES%NPLWKP_TOT(K),'0 0'
      ENDDO
!
! loop over spin, kpoints, bands
!
            DO ISP=1,WDES%ISPIN
            DO K=1,WDES%NKPTS

       CALL SETWDES(WDES,WDES1,K)
       NPL=WDES%NPLWKP_TOT(K)
       RNPL=NPL
       IREC=IREC+1
      
       io_begin
! write eigenvalues in real format
         DO J=1,WDES%NB_TOT
           EIG(J)=REAL(W%CELTOT(J,K,ISP),KIND=q)
         ENDDO
         DO J = 1,WDES%NB_TOT
         WRITE(1920,*) REAL(W%CELTOT(J,K,ISP),KIND=q),&
         REAL((W%CELTOT(J,K,ISP)*(0._q,-1._q)),KIND=q),W%FERTOT(J,K,ISP)
         ENDDO
       io_end
      
       DO J=1,WDES%NB_TOT
         CALL MRG_PW_BAND(WDES1, J, CW, W%CW(1,1,K,ISP))
         IREC=IREC+1
         do_io CRD(1:NPL)=CW(1:NPL)
         DO I =1,NPL
         WRITE(1919,*)REAL(CRD(I),KIND=q), &
                      & REAL((CRD(I)*(0._q,-1._q)),KIND=q)
         ENDDO
       ENDDO
      ENDDO 
      ENDDO

      DEALLOCATE(CW,CRD,EIG)
      CLOSE(1919)
      CLOSE(1920)
      
!     Ouptput FFT index information
      WRITE(IU0,*) 'OUTPUT_FFTINDEX: begin'
      OPEN(1815,FILE='FFTCOE',STATUS='NEW')
      WRITE(1815,*) GRID%NGX, GRID%NGY, GRID%NGZ
      WRITE(1815,*) LATT_INI%A
      WRITE(1815,*) LATT_INI%B
      WRITE(1815,*) WDES%NRPLWV,WDES%NKPTS,WDES%NB_TOT
      OPEN(1816,FILE='FFTIND',STATUS='NEW')
      DO LQINDKPT = 1, WDES%NKPTS
      DO LQINDPW  = 1, WDES%NGVECTOR(LQINDKPT)
      WRITE(1816,*) WDES%NINDPW(LQINDPW, LQINDKPT)
!      WRITE(1816,*) WDES%IGX(LQINDPW,LQINDKPT),WDES%IGY(LQINDPW,LQINDKPT),&
!                    WDES%IGZ(LQINDPW,LQINDKPT)
      ENDDO
      ENDDO
      CLOSE(1815)      
      CLOSE(1816)
      WRITE(IU0,*) 'OUTPUT_FFTINDEX: end'
      WRITE(IU0,*) 'Output projector in recipical space'
      
      
! Output phasefactor CREXP
      OPEN(1917,FILE='PHASE',STATUS='NEW')
      WRITE(1917,*)NONL_S%NIONS,0
      DO LQNK = 1,WDES%NKPTS 
      CALL PHASE(WDES,NONL_S,LQNK)
      DO LQNI = 1,NONL_S%NIONS
      LQGXDX=NONL_S%POSION(1,LQNI)
      LQGYDY=NONL_S%POSION(2,LQNI)
      LQGZDZ=NONL_S%POSION(3,LQNI)
      LQCREXP=EXP(CITPI*(LQGXDX*WDES%VKPT(1,LQNK)+&
              & LQGYDY*WDES%VKPT(2,LQNK)+LQGZDZ*WDES%VKPT(3,LQNK)))
      DO LQGN = 1,WDES%NGVECTOR(LQNK)
      WRITE(1917,*)REAL((1/(NONL_S%CREXP(LQGN,LQNI)*LQCREXP)),KIND=q),&
      REAL((1/(NONL_S%CREXP(LQGN,LQNI)*LQCREXP)*(0._q,-1._q)),KIND=q)
      ENDDO
      ENDDO
      ENDDO
      CLOSE(1917)
      
! Output Kpoint information
      OPEN(UNIT=14,FILE='KPOINTS',STATUS='OLD')
      READ(UNIT=14,FMT=*)
      READ(UNIT=14,FMT=*)
      READ(UNIT=14,FMT=*)
      READ(UNIT=14,FMT=*)LQNKPX,LQNKPY,LQNKPZ
      LQMAXK=LQNKPX*LQNKPY*LQNKPZ
      OPEN(1918,FILE='IKPT',STATUS='NEW')
      WRITE(1918,*)WDES%NKPTS,LQNKPX,LQNKPY,LQNKPZ
      DO LQL = 1,WDES%NKPTS
      WRITE(1918,'(3F20.15,4X,I10)')(WDES%VKPT(LQLL,LQL),LQLL=1,3),&
                                   NINT(LQMAXK*WDES%WTKPT(LQL))
      ENDDO
      CLOSE(1918)
     
! Output Symmetry information
      OPEN(1914,FILE='SYMM',STATUS='NEW')
      WRITE(1914,*)NROTK,NROT,'0'
      DO LQL = 1,NROTK
      DO LQLL = 1,3
      WRITE(1914,*)(IGRPOP(LQLL,LQLM,LQL),LQLM=1,3)
      ENDDO
      ENDDO
      CLOSE(1914)
      
! Output Qij information QION 
      OPEN(1916,FILE='QION',STATUS='NEW')      
      DO LQNT = 1,NONL_S%NTYP
      WRITE(1916,*)P(LQNT)%LMAX,0,0,0
      ENDDO
      
      DO LQNT = 1,NONL_S%NTYP
      DO LQL = 1 ,P(LQNT)%LMAX
      WRITE(1916,*)LQNT,LQL,P(LQNT)%LPS(LQL),0
      ENDDO
      ENDDO      
      
      DO LQNT = 1,NONL_S%NTYP
      DO LQL = 1 ,P(LQNT)%LMAX
      DO LQLL = LQL,P(LQNT)%LMAX
      WRITE(1916,*)LQNT,LQL,LQLL,P(LQNT)%QION(LQL,LQLL)
      ENDDO
      ENDDO
      ENDDO
      CLOSE(1916)
      
! Output Beta projector( |K| ) information  
!    05/04/2006
      OPEN(2916,FILE='BETAK',STATUS='NEW')      
      DO LQNT = 1,NONL_S%NTYP
      DO LQL = 1 ,P(LQNT)%LMAX
      WRITE(2916,'(2I6, 100F24.16)') LQNT,LQL, (P(LQNT)%PSPNL(I,LQL) ,I=1,LQNPSNL)
      ENDDO
      ENDDO      
      CLOSE(2916)

 ! Output Beta projector( |K| )'s max G length  information  
!    05/04/2006
      OPEN(2919,FILE='BETAKGMAX',STATUS='NEW')      
      DO LQNT = 1,NONL_S%NTYP      
      WRITE(2919,*) P(LQNT)%PSMAXN
      ENDDO      
      CLOSE(2919)

     
 ! Output reciprocal projector information QPROJ  
      OPEN(1915,FILE='BETA',STATUS='NEW')           
      IF (WDES%LSPIRAL) THEN 
      NSPINORS=2
      ELSE
      NSPINORS=1
      ENDIF
      DO LQISPINOR=1,NSPINORS
      DO LQNK = 1,WDES%NKPTS
      DO LQNT = 1,NONL_S%NTYP
      LQLMIND = 1
      DO LQL  = 1,P(LQNT)%LMAX
      LQLL = P(LQNT)%LPS(LQL)
      LQMMAX = 2*LQLL
      DO LQLM = 0,LQMMAX
      DO LQGN = 1,WDES%NGVECTOR(LQNK)
      WRITE(1915,*)NONL_S%QPROJ(LQGN,LQLMIND+LQLM,LQNT,LQNK,LQISPINOR)
      ENDDO
      ENDDO
      LQLMIND=LQLMIND + LQMMAX+1
      ENDDO
      ENDDO
      ENDDO
      ENDDO
      CLOSE(1915)     
      
      
!Output position
      OPEN(1921,FILE='POSE',STATUS='NEW')
      WRITE(1921,*)NONL_S%NTYP,NONL_S%NIONS,0,0
      LQNI=1;
      DO LQL =1,NONL_S%NTYP
      DO LQLL =1,NONL_S%NITYP(LQL)      
      WRITE(1921,'(4X,I10,3F20.15)')LQL,NONL_S%POSION(1,LQNI),&
      NONL_S%POSION(2,LQNI),NONL_S%POSION(3,LQNI)
      LQNI = LQNI+1
      ENDDO
      ENDDO
      CLOSE(1921)
      WRITE(IU0,*) 'Output projector finished!' 
      
!Output element symbol
      OPEN(1922,FILE='SPEC',STATUS='NEW')
      DO LQNT =1,NONL_S%NTYP
      WRITE(1922,'(1X,A2)') P(LQNT)%ELEMENT
      ENDDO
      CLOSE(1922)
      
      RETURN
      END 
