
##################################################################
# xcrysden script to show/rotate/zoom/print isosurface efficiently
# load lights and color parameters for isosurface
# Xiaofeng Qian, MIT (2006)
##################################################################


##################################################################
# procedure defined to generate a serie of png/jpg; #
# then use "vidcompress" or "vidcompress_mod"to generate mpeg movie #

proc printimage_png {ifile} {
    if {$ifile < 10} {
        set strfile "/tmp/Jpg/a0000$ifile.png"
    } elseif {$ifile < 100} {
        set strfile "/tmp/Jpg/a000$ifile.png"
    } elseif {$ifile < 1000} {
        set strfile "/tmp/Jpg/a00$ifile.png"
    } elseif {$ifile < 10000} {
        set strfile "/tmp/Jpg/a0$ifile.png"
    } else {
        set strfile "/tmp/Jpg/a$ifile.png"
    }
    
    scripting::printToFile $strfile windowdump
    return $strfile
}

proc printimage_jpg {ifile} {
    if {$ifile < 10} {
        set strfile "/tmp/Jpg/a0000$ifile.jpg"
    } elseif {$ifile < 100} {
        set strfile "/tmp/Jpg/a000$ifile.jpg"
    } elseif {$ifile < 1000} {
        set strfile "/tmp/Jpg/a00$ifile.jpg"
    } elseif {$ifile < 10000} {
        set strfile "/tmp/Jpg/a0$ifile.jpg"
    } else {
        set strfile "/tmp/Jpg/a$ifile.jpg"
    }
    
    scripting::printToFile $strfile windowdump
    return $strfile
}
##################################################################



##################################################################
# load xsf file; for multi-orbital xsf file,                    #
# one need to use "cutxsf" to make several one-orbital xsf file #
scripting::open --xsf data.xsf

# display in small window first, which make the rotation and rendering efficient #
scripting::displayWindow resize 400 400

# set own parameters #
set myParam(BACKGROUND)      {1.00 1.00 1.00}
set myParam(BALLF)           0.2

scripting::load_myParam
scripting::lighting On
scripting::displayMode3D BallStick
scripting::scalarField3D::load

# hide isosurface control panel -->  # 
# this hide method still will give icon on the left-bottom of image#
# however, you can opend the panel and modify from this method #
IsoControl_Hide

# another way to hide isosurface control panel -->  # 
# Good: it won't give any icon on the image #
# however there is no panel you can open to modify #
# therefore this will be the final stadge to generate images #
# wm withdraw .iso #

scripting::scalarField3D::configure \
    -isosurface 1 \
    -interpolation_degree 3 \
    -isolevel 0.025 \
    -plusminus 1

scripting::buildCrystal 1 1 1
scripting::zoom 2 1
scripting::rotate x -90 
scripting::scalarField3D::render
############################################################


############################################################
#  setup GL color/lightmodel parameters for isosurface #
#  Tcl/isoControl.tcl: line 1429 #
xc_setGLparam isonormal -what isosurf_neg

#  Tcl/glLight.tcl #
xc_setGLparam lightmodel -two_side_iso 1
xc_setGLparam material -what isosurf pos front -shininess {100} \
    -specular {1.0 1.0 1.0 1.0} -ambient {1.0 0.8 0.0 1.0} -diffuse {1.0 0.8 0.0 1.0}
xc_setGLparam material -what isosurf pos back  -shininess {100} \
    -specular {1.0 1.0 1.0 1.0} -ambient {1.0 0.8 0.0 1.0} -diffuse {1.0 0.8 0.0 1.0}
xc_setGLparam material -what isosurf neg front -shininess {100} \
    -specular {1.0 1.0 1.0 1.0} -ambient {0.3 0.4 1.0 1.0} -diffuse {0.3 0.4 1.0 1.0}
xc_setGLparam material -what isosurf neg back  -shininess {100} \
    -specular {1.0 1.0 1.0 1.0} -ambient {0.3 0.4 1.0 1.0} -diffuse {0.3 0.4 1.0 1.0}
############################################################

############################################################
# load lights from ~/.xcrysden directory #
# glLight: line 267 #
glLight:load {~/.xcrysden/my_lights_4.lights}

# glLight: line 162 #
glLight:update .mesa 1
glLight:update .mesa 2
glLight:update .mesa 3
glLight:update .mesa 4
############################################################

# update #
.mesa render



# print png out #
# one way to generate high-resolution eps/png/jpg file for publication #
scripting::displayWindow resize 1600 1600
scripting::printToFile test.eps windowdump

# change window size back to small setting
scripting::displayWindow resize 400 400



